@@ -322,6 +322,85 @@ class AzureML(BackendBase):
         results_by_aml = {"Azure ML": results}
         return results_by_aml
 
+    def get_active_jobs(self):
+        ''' return a list of job_id's running on this instance of Azure Batch '''
+        mongo = self.store.get_mongo()
+
+        filter_dict = {
+            "username": self.username,
+            "compute": "aml",
+            "job_status": {
+                "$nin": ["created", "completed"]
+            }
+        }
+
+        fields_dict = {"job_id": 1, "service_info_by_node": 1, "service_job_info": 1}
+
+        job_records = mongo.get_info_for_jobs(filter_dict, fields_dict)
+
+        return job_records
+
+    def cancel_runs_by_user(self, box_name):
+        '''
+        Args:
+            box_name: the name of the box the runs ran on (pool service)
+        Returns:
+            cancel_results: a list of kill results records 
+                (keys: workspace, run_name, exper_name, killed, status, before_status)
+        '''
+        cancel_results = []
+
+        # get list of active jobs from batch
+        active_jobs = self.get_active_jobs()
+        console.diag("after get_active_jobs()")
+
+        if active_jobs:
+            for job_record in active_jobs:
+                # watch out for older jobs that didn't have service_job_info/service_info_by_node properties
+                service_job_info = utils.safe_value(job_record, "service_job_info")
+                service_info_by_node = utils.safe_value(job_record, "service_info_by_node")
+
+                if service_job_info and service_info_by_node:
+                    job_id = job_record["job_id"]
+                    cancel_result = self.cancel_job(service_job_info, service_info_by_node)
+                    for _, node_result in cancel_result.items():
+                        cancel_results.append(node_result)
+
+        return cancel_results
+
+    def cancel_runs_by_names(self, workspace, run_names, box_name):
+        '''
+        Args:
+            workspace: the name of the workspace containing the run_names
+            run_names: a list of run names
+            box_name: the name of the box the runs ran on (pool service)
+        Returns:
+            cancel_results: a list of kill results records 
+                (keys: workspace, run_name, exper_name, killed, status, before_status)
+        '''
+
+        # our strategy for this API: 
+        #   - use the XT controller to kill specified runs (when controller is available)
+        #   - use batch_client "cancel node" if controller not available
+
+        # we build service-based box names to have 3 parts
+        job_id, service_name, node_index = box_name.split("-")
+        active_jobs = self.get_active_jobs()
+        cancel_results = []
+        if active_jobs:
+            for job_record in active_jobs:
+                # watch out for older jobs that didn't have service_job_info/service_info_by_node properties
+                service_info_by_node = utils.safe_value(job_record, "service_info_by_node")
+                
+                if service_info_by_node:
+                    for node, node_service_info in service_info_by_node.items():
+                        if node_service_info.get("run_name") in run_names:
+                            cancel_result = self.cancel_node(node_service_info)
+                            cancel_results.append(cancel_result)
+
+        return cancel_results
+
+
     def build_env_vars(self, workspace, aml_ws_name, xt_exper_name, aml_exper_name, run_name, job_id, 
         compute_target, username, description, aggregate_dest, node_id, args):
 
@@ -958,25 +1037,30 @@ class AzureML(BackendBase):
 
         for node_id, node_info in service_info_by_node.items():
             result = self.cancel_node(node_info)
-            result_by_node[node_id] = result
+            if result is not None:
+                result_by_node[node_id] = result
 
         return result_by_node
 
     # API call
     def cancel_node(self, service_node_info):
-        run = self.get_node_run(service_node_info)
-
-        service_status = run.get_status()
-        simple_status = self.get_simple_status(service_status)
-        cancelled = False
-
-        if simple_status != "completed":
-            
-            run.cancel()
+        result = None
+        if "aml_run_id" in service_node_info:
+            run = self.get_node_run(service_node_info)
 
             service_status = run.get_status()
             simple_status = self.get_simple_status(service_status)
-            cancelled = (service_status == "completed")     
+            before_status = simple_status
+            cancelled = False
+
+            if simple_status != "completed":
+                
+                run.cancel()
+
+                service_status = run.get_status()
+                simple_status = self.get_simple_status(service_status)
+                cancelled = (service_status == "completed")     
+
+            result = {"cancelled": cancelled, "service_status": service_status, "simple_status": simple_status}
 
-        result = {"cancelled": cancelled, "service_status": service_status, "simple_status": simple_status}
         return result
