@@ -14,6 +14,7 @@ from xtlib import errors
 from xtlib import scriptor
 from xtlib import pc_utils
 from xtlib import constants
+from xtlib import file_utils
 
 from .backend_interface import BackendInterface
 
@@ -26,6 +27,13 @@ class BackendBase(implements(BackendInterface)):
         self.is_windows = False
         self.fn_wrapped = None
 
+    # API 
+    def get_name(self):
+        '''
+        This method is called return the name of the backend service.
+        '''
+        pass
+
     # API 
     def adjust_run_commands(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, args):
         '''
@@ -93,7 +101,7 @@ class BackendBase(implements(BackendInterface)):
         raise Exception("backend API function not implemented: cancel_runs_by_user")
 
     # common code
-    def append(self, cmds, cmd, echo_title="running", echo_before=False, echo_after=False, expand=False):
+    def append(self, cmds, cmd, echo_before=False, echo_after=False, expand=False):
         
         echo = "@echo" if self.is_windows else "echo"
 
@@ -104,17 +112,35 @@ class BackendBase(implements(BackendInterface)):
             cmd = "@" + cmd
 
         if echo_before:
-            cmds.append("{} 'before {}: {}'".format(echo, echo_title, cmd))
+            cmds.append("{} running: {}".format(echo, cmd))
 
         cmds.append(cmd)
 
         if echo_after:
-            cmds.append("{} 'after {}: {}'".format(echo, echo_title, cmd))
+            cmds.append("{} after: {}".format(echo, cmd))
 
     def get_activate_cmd(self):
 
         setup_def = self.config.get_setup_from_target_def(self.compute_def)
-        activate_cmd = setup_def["activate"] if setup_def and "activate" in setup_def else None
+
+        if pc_utils.is_windows:
+            activate_cmd = utils.safe_value(setup_def, "activate")
+        else:
+            # Attempting to activate the Conda shell from within a bash script
+            # fails, with Conda saying that the bash environment has not
+            # been correctly initialized to use Conda.
+            # This thread https://stackoverflow.com/questions/34534513/calling-conda-source-activate-from-bash-script
+            # eventually led me to the following command which is taken
+            # from the lines of bash script that Conda appends to your
+            # .bashrc file upon installation. This command is what
+            # allows you to activate the Conda environment within a
+            # bash shell. It returns a script generated by Conda
+            # which is executed, and which stes up the conda
+            # activate / deactivate commands in the encironment.
+            conda_shell_bash_hook_cmd = 'eval "$(conda shell.bash hook)"'
+            activate_cmd = utils.safe_value(setup_def, "activate")
+            activate_cmd = "{} && {}".format(
+                conda_shell_bash_hook_cmd, activate_cmd)
 
         return activate_cmd
 
@@ -223,7 +249,7 @@ class BackendBase(implements(BackendInterface)):
                 cmds += sub_cmds
 
             cmd = "{} download /{}/{} {}".format(xt_path, container_name, blob_path, dest_dir)
-            self.append(cmds, cmd, "running: ", True, True)
+            self.append(cmds, cmd, True, True)
 
         return cmds        
 
@@ -341,14 +367,18 @@ class BackendBase(implements(BackendInterface)):
 
     def add_first_cmds(self, cmds, script_name, change_dir):
 
+        if not pc_utils.is_windows():
+            self.append(cmds, "#!/bin/sh")
+
         self.append(cmds, 'echo ----- START of XT-level processing -----')
         self.append(cmds, "echo running: " + script_name)
 
         self.append(cmds, 'echo initial cwd: {}'.format("%cd%" if self.is_windows else "$PWD"))
         cwd = utils.get_controller_cwd(self.is_windows, False)
+        cwd = file_utils.fix_slashes(cwd, is_linux=not self.is_windows)
 
         if self.is_windows:
-            self.append(cmds, "mkdir {}".format(cwd), echo_before=True)
+            self.append(cmds, "mkdir {} 2>nul".format(cwd), echo_before=True)
         else:
             self.append(cmds, "mkdir {} -p".format(cwd), echo_before=True)
 
@@ -493,7 +523,7 @@ class BackendBase(implements(BackendInterface)):
                 cmd += ' {}'.format(pp)
 
             cmd += " > __pip_install__.log"
-            self.append(cmds, cmd, "running: ", True, True)
+            self.append(cmds, cmd, True, True)
 
     def add_conda_packages(self, cmds, conda_packages):
         if conda_packages:
@@ -503,7 +533,7 @@ class BackendBase(implements(BackendInterface)):
                 cmd += ' {}'.format(cp)
 
             cmd += " > __conda_install__.log"
-            self.append(cmds, cmd, "running: ", True, True)
+            self.append(cmds, cmd, True, True)
 
     def add_setup_cmds(self, cmds, pip_freeze, args):
 
@@ -537,3 +567,29 @@ class BackendBase(implements(BackendInterface)):
             self.append(cmds, '@set PATH=%HOME%/.local/bin;%PATH%')
         else:
             self.append(cmds, 'export PATH=$HOME/.local/bin:$PATH')
+
+    # API call
+    def get_node_status(self, service_node_info):
+        pass
+
+    # API call
+    def read_log_file(self, service_node_info, log_name, start_offset=0, end_offset=None, 
+        encoding='utf-8', use_best_log=True):
+        pass
+
+    # API call
+    def get_simple_status(self, status):
+        # translates an Philly status to a simple status (queued, running, completed)
+        pass
+
+    # API call
+    def cancel_job(self, service_job_info, service_info_by_node):
+        pass
+    
+    # API call
+    def cancel_node(self, service_node_info):            
+        pass
+
+    # API call
+    def get_service_queue_entries(self, service_node_info):
+        pass
\ No newline at end of file
