@@ -13,6 +13,18 @@ class BackendInterface(Interface):
         '''
         pass
 
+    # API call
+    def get_name(self):
+        '''
+        Returns:
+            name of the service (e.g., batch or aml)
+
+        Description:
+            This method is called to get the name of the backend service.
+        '''
+        pass
+    
+    # API call
     def adjust_run_commands(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, args):
         '''
         Args:
@@ -32,6 +44,7 @@ class BackendInterface(Interface):
         '''
         pass
 
+    # API call
     def submit_job(self, job_id, job_runs, workspace, compute_def, resume_name, 
             repeat_count, using_hp, runs_by_box, experiment, snapshot_dir, controller_scripts, args):
         '''
@@ -58,6 +71,7 @@ class BackendInterface(Interface):
         '''
         pass
 
+    # API call
     def get_client_cs(self, service_node_info):
         '''
         Args:
@@ -71,6 +85,7 @@ class BackendInterface(Interface):
         '''
         pass
 
+    # API call
     def view_status(self, run_name, workspace, job, monitor, escape_secs, auto_start, 
             stage_flags, status, max_finished):
         '''
@@ -92,6 +107,7 @@ class BackendInterface(Interface):
         '''
         pass
 
+    # API call
     def provides_container_support(self):
         '''
         Returns:
@@ -99,6 +115,7 @@ class BackendInterface(Interface):
         '''
         pass
 
+    # API call
     def cancel_runs_by_names(self, workspace, run_names, box_name):
         '''
         Args:
@@ -114,6 +131,7 @@ class BackendInterface(Interface):
         '''
         pass
 
+    # API call
     def cancel_runs_by_job(self, job_id, runs_by_box):
         '''
         Args:
@@ -128,6 +146,7 @@ class BackendInterface(Interface):
         '''
         pass
 
+    # API call
     def cancel_runs_by_user(self, box_name):
         '''
         Args:
@@ -141,3 +160,98 @@ class BackendInterface(Interface):
         '''
         pass
 
+    # API call
+    def read_log_file(self, service_node_info, log_name, start_offset=0, end_offset=None, 
+        encoding='utf-8', use_best_log=True):
+        '''
+        Args:
+            service_node_info: a service-defined dictionary that identifies a node assoicated with a job
+            log_name: the name of a specific log file to read (stdout, stderr, log70, etc.)
+            start_offset: starting offset to read from in log file
+            end_offset: last offset of log file to read (if None, rest of file is read)
+            encoding: how to encode bytes to text for log content
+            use_best_log: when True, backend will continually choose the best log to be return
+        Returns a dictionary with following keys:
+            new_text  (the specified subset of the file)
+            simple_status (a string, one of: queued, running, completed) 
+            service_status (the service specific status value)
+            log_name (the name of the returned log file)
+            next_offset (the next streaming offset to be used for the returned log file)
+
+        Description:
+            reads the specified subset of the specified log file, for the node identified by
+            service_node_info.
+       '''
+        pass
+
+    # API call
+    def get_simple_status(self, status):
+        '''
+        Args:
+            status: the backend-specific status value
+        Returns:
+            returns a simple status from one of these values: queued, running, completed
+
+        Description:
+            translated the service status to a simple status.
+        '''
+        pass
+
+    # API call
+    def cancel_job(self, service_job_info, service_info_by_node):
+        '''
+        Args:
+            service_job_info: backend-specific dict to identify the job
+            service_info_by_node: dict of backend-specific dict to identify each node
+
+        Returns:
+            result_by_node: a dict of cancel result records: {cancelled: bool, server_status, simple_status}
+            
+        Description:
+            cancels each node of the specified job, and if appropriate for the service, cancels the job itself
+        '''
+        pass
+
+    # API call
+    def cancel_node(self, service_node_info):            
+        '''
+        Args:
+            service_node_info: backend-specific dict to identify the node
+
+        Returns:
+            result_record: {cancelled: bool, server_status, simple_status}
+            
+        Description:
+            cancels the run/task running on the specified node.
+        '''
+        pass
+
+    # API call
+    def get_node_status(self, service_node_info):
+        '''
+        Args:
+            service_node_info: backend-specific dict to identify the node
+
+        Returns:
+            service-specific status (string)
+            
+        Description:
+            returns the status of the specified node.
+        '''
+        pass
+
+    # API call
+    def get_service_queue_entries(self, service_node_info):
+        '''
+        Args:
+            service_node_info: backend-specific dict to identify the node
+
+        Returns:
+            a list of records: first is the currently executing job and 
+            the others are the jobs in the queue, waiting to run.
+            record keys: current (bool), name (name job entry)
+            
+        Description:
+            gathers and returns the list of current + queue entries.
+        '''
+        pass
