@@ -170,140 +170,15 @@ class PoolBackend(BackendBase):
         fn_src_zip = file_utils.path_join(cwd_dir, constants.CODE_ZIP_FN)
 
         fn_entry = psm_client.enqueue(team, job_id, node_id, fn_src_zip)
+        run_data = run_data_list[0]
 
         service_node_info = {"fn_entry": fn_entry, "box_addr": box_addr, "box_os": box_os, 
-            "box_name": box_name, "job_id": job_id}
+            "box_name": box_name, "job_id": job_id, "run_name": run_data["run_name"]}
 
         fb.feedback("submitted", is_final=True) 
 
         return service_node_info
 
-    # def old_run_job_on_box(self, job_id, run_data_list, box_index, box_info, app_info, pool_info,  
-    #         resume_name=None, repeat=None, using_hp=None, exper_name=None, snapshot_dir=None, args=None):
-    #     '''
-    #     copy the startup script to the specified box (using SCP) and run it (using SSH)
-    #     '''
-    #     box_name = box_info.box_name
-    #     workspace = args["workspace"]
-    #     #console.print("box_name=", box_name, ", box_index=", box_index)
-
-    #     box_addr = box_info.address
-    #     is_local = box_addr == "localhost"
-
-    #     src_windows = pc_utils.is_windows()
-    #     dst_windows = box_info.box_os == "windows"
-
-    #     hold_open = args["hold"]
-    #     run_data = run_data_list[0]
-    #     run_name = run_data["run_name"]
-    #     ws = args["workspace"]
-
-    #     # kill any running instances of the controller on the box (those that didn't shut down cleanly)
-    #     self.client.cancel_thru_os(box_name, False)
-
-    #     # send over files to box
-    #     dest_dir = utils.get_controller_cwd(dst_windows, is_local=False)
-    #     fn_bootstrap = file_utils.path_join(dest_dir, os.path.basename(self.fn_wrapped))
-
-    #     visible = args["show_controller"]
-    #     hold_open = True
-
-    #     if is_local:
-    #         # create the directory on the local machine
-    #         os.makedirs(os.path.expanduser(dest_dir), exist_ok=True)
-
-    #         # LOCAL box
-    #         # COPY the snapshot directory (localhost)
-    #         dest_dir = os.path.expanduser(dest_dir)
-    #         file_utils.copy_tree(snapshot_dir, dest_dir, omit_dirs="__pycache__")
-    #         fb.feedback("files copied to box")
-    #         fb.feedback("{}/{}".format(ws, run_name))
-
-    #         # Make the wrapper script executable
-    #         if not src_windows:
-    #             wrapper_cmd = "{}/{}".format(
-    #                 dest_dir,
-    #                 constants.FN_WRAPPED_CMDS)
-    #             st = os.stat(wrapper_cmd)
-    #             os.chmod(wrapper_cmd, st.st_mode | stat.S_IEXEC)
-
-    #         # start controller  (localhost)
-    #         fn_log = os.path.expanduser("~/.xt/controller.log")
-    #         fn_bootstrap = os.path.expanduser(fn_bootstrap)
-
-    #         if hold_open and visible:
-    #             if src_windows:
-    #                 os.system("start cmd /K " + fn_bootstrap)
-    #             else:
-    #                 process_utils.start_async_run_detached(fn_bootstrap, os.path.expanduser(dest_dir), fn_log, visible=visible)
-    #         else:            
-    #             process_utils.start_async_run_detached(fn_bootstrap, os.path.expanduser(dest_dir), fn_log, visible=visible)
-
-    #     else:
-    #         # REMOTE box
-    #         # copy the .zip file using scp
-    #         src_dir = os.path.expanduser(constants.CWD_DIR)
-    #         fn_src = file_utils.path_join(src_dir, constants.CODE_ZIP_FN)
-    #         fn_src = os.path.expanduser(fn_src)
-
-    #         fn_dest = file_utils.path_join(dest_dir, constants.CODE_ZIP_FN)
-
-    #         # create the directory on the pool machine thru SSH
-    #         cmd = "mkdir -p {}".format(dest_dir)
-    #         fb.feedback("[md")
-    #         err_code2, output2 = process_utils.sync_run_ssh(self, box_addr, cmd)
-
-    #         # run the COPY thru SCP
-    #         fb.feedback("cp")
-    #         process_utils.scp_copy_file_to_box(self, box_addr, fn_src, fn_dest)
-
-    #         # unzip the file
-    #         # 1. we need the remote box expanded fn_dest
-    #         err_code, output = process_utils.sync_run_ssh(self, box_addr, "ls " + fn_dest)
-    #         full_fn_dest = output.strip().split("\n")[-1]
-
-    #         #cmd = ''' python -c \\"import zipfile;  zip = zipfile.ZipFile('{}', 'r'); zip.extractall('{}')\\" '''.format(full_fn_dest, os.path.dirname(full_fn_dest))
-    #         py_cmd = "import zipfile; zip = zipfile.ZipFile('{}', 'r'); zip.extractall('{}')".format(full_fn_dest, os.path.dirname(full_fn_dest))
-    #         ssh_cmd = 'python -c "{}"'.format(py_cmd)
-    #         fb.feedback("unzip]")
-    #         err_code2, output2 = process_utils.sync_run_ssh(self, box_addr, ssh_cmd)
-
-    #         fb.feedback("files copied to box")
-    #         fb.feedback("{}/{}".format(ws, run_name))
-
-    #         # start the run
-    #         run_in_cmd_window = visible
-
-    #         if run_in_cmd_window:
-    #             # for debugging, open a new cmd window and watch real-time output from remote box
-    #             cmd = "bash --login " + fn_bootstrap
-    #             ssh_parts = process_utils.make_ssh_cmd_parts(box_addr, cmd)
-    #             ssh_cmd = " ".join(ssh_parts)
-
-    #             local_cwd = utils.get_controller_cwd(True, is_local=True)
-    #             fn_ssh = os.path.join(local_cwd, "run_ssh.bat")
-    #             fn_log = os.path.join(local_cwd, "run_ssh.log")
-    #             file_utils.write_text_file(fn_ssh, ssh_cmd)
-
-    #             if hold_open and visible:
-    #                 os.system("start cmd /K " + fn_ssh)
-    #             else:
-    #                 process_utils.start_async_run_detached(fn_ssh, local_cwd, fn_log, visible=True)
-    #         else:
-    #             # normal run 
-    #             #cmd = "bash --login " + fn_bootstrap
-    #             cmd = 'nohup bash --login {} </dev/null > ~/.xt/controller.log 2>&1 &'.format(fn_bootstrap) 
-
-    #             process_utils.sync_run_ssh(self, box_addr, cmd)
-
-    #     fb.feedback("submitted", is_final=True)  
-        
-    #     monitor_url = None
-
-    #     service_node_info = {"job_id": job_id, "node_index": box_index, "box_name": box_name, "monitor_url": monitor_url}
-
-    #     return service_node_info
-
     def get_env_vars_for_box(self, box_name, box_info, nodex_index, box_secret):
         #box_secrets.set_secret(box_name, box_secret)
 
@@ -378,28 +253,22 @@ class PoolBackend(BackendBase):
             cancel_results: a list of kill results records 
                 (keys: workspace, run_name, exper_name, killed, status, before_status)
         '''
-        cancel_results = None
 
-        try:
-            # connect to specified box
-            first_run_name = run_names[0]
+        # our strategy for this API: 
+        #   - use the XT controller to kill specified runs (when controller is available)
+        #   - use batch_client "cancel node" if controller not available
 
-            cs, box_secret = run_helper.get_client_cs(self.core, workspace, first_run_name)
-            if not cs:
-                console.print("could not find client connection string for run: {}/{}".format(workspace, first_run_name))
-            else:
-                # add workspace to run_names
-                run_names = [workspace + "/" + run_name for run_name in run_names]
-
-                # send request to controller via the client service
-                with XTClient(self.config, cs, box_secret) as xtc:
-                    if xtc.connect():
-                        cancel_results = xtc.cancel_runs(run_names)
-                    else:
-                        console.print("couldn't connect to controller for {}".format(box_name))
-        except BaseException as ex:
-            errors.report_exception(ex)
-            pass
+        # we build service-based box names to have 3 parts
+        active_jobs = self.get_active_jobs()
+        cancel_results = []
+        if active_jobs:
+            for job_record in active_jobs:
+                service_job_info = job_record["service_job_info"]
+                service_info_by_node = job_record["service_info_by_node"]
+                for node, node_service_info in service_info_by_node.items():
+                    if node_service_info.get("run_name") in run_names:
+                        cancel_result = self.cancel_node(node_service_info)
+                        cancel_results.append(cancel_result)
 
         return cancel_results
 
@@ -443,7 +312,16 @@ class PoolBackend(BackendBase):
             cancel_results: a list of kill results records 
                 (keys: workspace, run_name, exper_name, killed, status, before_status)
         '''
-        cancel_results =  self.cancel_runs_by_property("username", self.username, box_name)
+        active_jobs = self.get_active_jobs()
+        cancel_results = []
+        if active_jobs:
+            for job_record in active_jobs:
+                service_job_info = job_record["service_job_info"]
+                service_info_by_node = job_record["service_info_by_node"]
+                for node, node_service_info in service_info_by_node.items():
+                    cancel_result = self.cancel_node(node_service_info)
+                    cancel_results.append(cancel_result)
+
         return cancel_results
 
     def get_psm_client(self, service_node_info):
@@ -554,4 +432,23 @@ class PoolBackend(BackendBase):
         if current:
             queue.insert(0, {"name": current, "current": True})
 
-        return queue
\ No newline at end of file
+        return queue
+
+    def get_active_jobs(self):
+        ''' return a list of job_id's running on this instance of Azure Batch '''
+        mongo = self.client.store.get_mongo()
+
+        filter_dict = {
+            "username": self.username,
+            "compute": "local",
+            "ws_name": self.config.get("general", "workspace"),
+            "job_status": {
+                "$nin": ["created", "completed"]
+            }
+        }
+
+        fields_dict = {"job_id": 1, "service_info_by_node": 1, "service_job_info": 1}
+
+        job_records = mongo.get_info_for_jobs(filter_dict, fields_dict)
+
+        return job_records
