@@ -140,7 +140,7 @@ def download_run(store, ws_name, run_name, dest_dir):
     before_dir = dest_dir + "/before"
     before_files = store.download_files_from_job(job_id, "before/**", before_dir)
 
-    zip_files = unzip_before_if_found(before_dir + "/code")
+    zip_files = unzip_before_if_found(before_dir + "/code", silent=True)
     if zip_files:
         files += zip_files
     else:
