@@ -268,6 +268,22 @@ class CmdCore():
 
         context = Bag()
 
+        # transfer the set of properties
+        context.update_job_stats = self.config.get("mongo", "update-job-stats")
+        context.update_run_stats = self.config.get("mongo", "update-run-stats")
+        context.add_log_records = self.config.get("mongo", "add-log-records")
+        context.use_simple_scheduling = self.config.get("mongo", "use-simple-scheduling")
+
+        # simple scheduling
+        total_run_count = args["total_run_count"]
+        node_count = args["node_count"]
+        runs_per_node = math.ceil(total_run_count/node_count)
+        runs_last_node = total_run_count - (runs_per_node * (node_count-1))
+        node_run_count = runs_per_node if node_index < node_count-1 else runs_last_node
+
+        context.first_run_index = runs_per_node*node_index
+        context.last_run_index = context.first_run_index + node_run_count - 1
+
         context.ws = workspace
         context.working_dir = working_dir
         context.exper_name = exper_name
