@@ -4,8 +4,8 @@
 #
 # constants.py: constant strings and values share among multiple modules
 # sync this version with setup.py, CHANGELOG.md
-BUILD = "version: 0.0.199, build: Apr-30-2020"
-
+BUILD = "version: 0.0.203, build: May-15-2020"
+    
 INFO_CONTAINER = "xt-store-info"
 INFO_DIR = "__info__"
 STORAGE_VERSION = "1"
@@ -125,3 +125,6 @@ PSMLOG = "psm.log"
 PSM_LOGDIR = "~/.xt/psm_logs"
 CURRENT_RUNNING_ENTRY = "_current_running_entry_.txt"
 
+# hyperparameter YAML property names
+HPARAM_DIST = "hyperparameter-distributions"
+HPARAM_RUNSET = "hyperparameter-runset"
