@@ -127,6 +127,13 @@ class XTController(rpyc.Service):
         self.is_aml = is_aml
         self.restarting = False
         self.restart_delay = None
+        self.mri = None
+
+        # if previous controller log file exists, remove it
+        fn = os.path.expanduser(constants.FN_CONTROLLER_EVENTS)
+        if os.path.exists(fn):
+            console.print("removing previous CONTROLLER LOG file")
+            os.remove(fn)
 
         self.reset_state(start_queue_worker=True)
 
@@ -227,7 +234,13 @@ class XTController(rpyc.Service):
         parent_run_name = self.mrc_context.run_name
 
         self.job_id = self.mrc_context.job_id
+
+        # cache store for later use (don't want storge/mongo traffic needed for subsequent creations)
         store = store_from_context(self.mrc_context)
+        self.store = store
+        mongo = store.mongo
+
+        self.use_simple_scheduling = self.mrc_context.use_simple_scheduling
 
         was_queued = []    # list of runs that were queued before we were restarted
 
@@ -236,10 +249,17 @@ class XTController(rpyc.Service):
         cmd_parts = context.cmd_parts
         aml_run = False
 
+        # support for simple scheduling
+        self.parent_run_name = parent_run_name
+        self.next_child_id = 1
+        self.first_run_index = context.first_run_index
+        self.last_run_index = context.last_run_index
+
         self.write_connect_info_to_job(store, self.job_id)
         
         # controls allocation of child runs 
-        self.mri = MongoRunIndex(store.mongo, self.job_id, parent_run_name, self.node_id)
+        if not self.use_simple_scheduling:
+            self.mri = MongoRunIndex(store.mongo, self.job_id, parent_run_name, self.node_id)
 
         # queue up first job
         self.queue_job_core(context, cmd_parts, aml_run=aml_run)
@@ -284,7 +304,7 @@ class XTController(rpyc.Service):
             return len(self.queue)
 
     def on_shutdown(self, context):
-        console.print("processing shutdown request in queue thread...")
+        console.print("-----> on_shutdown: starting")
 
         #remove the cert file
         fn_server_cert = os.path.expanduser(constants.FN_SERVER_CERT)
@@ -295,16 +315,19 @@ class XTController(rpyc.Service):
         logging.shutdown()        
 
         if context:
-            store = store_from_context(context)
+            store = self.store
             node_num = os.getenv("XT_NODE_ID")[4:]
 
             # write XT event log to job store AFTER
             fn = os.path.expanduser(constants.FN_CONTROLLER_EVENTS)
-            console.print("fn=", fn, ", exists=", os.path.exists(fn))
+            console.print("fn=", fn, ", exists=", os.path.exists(fn))   
 
             if os.path.exists(fn):
                 fn_store = "node-{}/after/{}".format(node_num, os.path.basename(fn))
+
+                console.print("starting to upload CONTROLLER log: {} => {}".format(fn, fn_store))
                 store.upload_file_to_job(context.job_id, fn_store, fn)
+                console.print("finished upload of CONTROLLER log")
 
                 # bug workaround - the above shutdown doesn't free the file, so we
                 # just make sure to delete the file on startup of next controller run
@@ -324,6 +347,9 @@ class XTController(rpyc.Service):
         time.sleep(2)    # wait for 2 secs for any bg thread cleanup
         console.print("calling os._exit(0)...")
 
+        print("XT controller: mongo call stats")
+        self.store.mongo.print_call_stats()
+
         # os._exit will exit all threads without running 'finally' blocks 
         # sys.exit will exit current thread only, but run 'finally' blocks for a cleaner exit
         os._exit(0)
@@ -344,7 +370,7 @@ class XTController(rpyc.Service):
             if self.runs:
                 first_run = list(self.runs.values())[0]
                 context = first_run.context
-                store = store_from_context(context)
+                store = self.store
 
                 for job_id, alive in self.running_jobs.items():
                     if alive:
@@ -352,6 +378,7 @@ class XTController(rpyc.Service):
 
                         # tell mongo this job node is exiting
                         console.diag("calling MONGO job_node_exit: job_id={}".format(job_id))
+
                         mongo = store.get_mongo()
                         mongo.job_node_exit(job_id)
             else:
@@ -377,11 +404,11 @@ class XTController(rpyc.Service):
                 break
 
         # AFTER potentially starting a run, see if we are idle
-        names = self.get_running_names()
-        running_count = len(names)
+        alive = self.get_alive_names()
+        alive_count = len(alive)
         #print("queue_check: running_count={}, names={}".format(running_count, names))
 
-        if running_count == 0:
+        if alive_count == 0:
             self.on_idle()
 
     def process_top_of_queue(self, running_count):
@@ -416,10 +443,20 @@ class XTController(rpyc.Service):
             
             # should parent spawn new child?
             context = run_info.context
-            store = store_from_context(context)
+            store = self.store
 
             #entry = store.mongo.get_next_run_index(run_info.job_id, self.node_id, self.first_call)
-            entry = self.mri.get_next_child_run()
+            if self.mri:
+                entry = self.mri.get_next_child_run()
+            else:
+                # simple scheduler (static, with no run-level restart support)
+                ri = self.first_run_index + self.next_child_id - 1
+                if ri > self.last_run_index:
+                    entry = None
+                else:
+                    run_name = self.parent_run_name + "." + str(self.next_child_id)
+                    self.next_child_id += 1
+                    entry = {"run_index": ri, "run_name": run_name, "node_id": self.node_id, "status": "unstarted", "job_id": self.job_id}
 
             if entry:
                 run_index = entry["run_index"]
@@ -469,7 +506,7 @@ class XTController(rpyc.Service):
 
         # create a parent log event for "spawning"
         context = run_info.context
-        store = store_from_context(context)
+        store = self.store
         store.log_run_event(context.ws, run_name, "status-change", {"status": "spawning"})   # , is_aml=self.is_aml)
 
         # spawn child run from template
@@ -523,7 +560,7 @@ class XTController(rpyc.Service):
         # update context with new cmd
         context.cmd_parts = utils.cmd_split(cmd)
 
-        store = store_from_context(context)
+        store = self.store
 
         console.print("spawn_child for parent=", parent.run_name)
 
@@ -579,7 +616,8 @@ class XTController(rpyc.Service):
 
         child_info = RunInfo(child_name, context.ws, cmd_parts, context.run_script, 
             None, context, "running", True, parent_name=parent.run_name, mirror_close_func = self.stop_mirror_worker, 
-            aml_run=child_aml_run, node_id=self.node_id, run_index=run_index, mri=self.mri, mri_entry=entry)
+            aml_run=child_aml_run, node_id=self.node_id, run_index=run_index, mri=self.mri, mri_entry=entry, 
+            store=self.store)
 
         utils.print_elapsed(spawn_start, "SPAWN CHILD")
 
@@ -630,6 +668,8 @@ class XTController(rpyc.Service):
         else:
             run_info.is_wrapped_up = True
 
+        console.print("end of exit handler")
+
     def on_connect(self, conn):
         # code that runs when a connection is created
         # (to init the service, if needed)
@@ -813,10 +853,10 @@ class XTController(rpyc.Service):
 
         run_info = RunInfo(run_name, context.ws, cmd_parts, run_script, context.repeat, context, "queued", True, 
             parent_name=None, parent_prep_needed=parent_prep_needed, mirror_close_func = self.stop_mirror_worker, aml_run=aml_run,
-            node_id=self.node_id, run_index=None)
+            node_id=self.node_id, run_index=None, store=self.store)
 
         # log run QUEUED event 
-        store = store_from_context(context)
+        store = self.store
         store.log_run_event(context.ws, run_name, "queued", {})   # , is_aml=self.is_aml)
 
         # queue job to be run
@@ -902,7 +942,7 @@ class XTController(rpyc.Service):
         run_name = run_info.run_name
 
         # download files from STORE to rundir
-        store = store_from_context(context)
+        store = self.store
         job_id = context.job_id 
 
         if self.is_aml:
@@ -1215,7 +1255,7 @@ class XTController(rpyc.Service):
 
         # log run KILLED event 
         context = run_info.context
-        store = store_from_context(context)
+        store = self.store
 
         if not for_restart:
             store.log_run_event(context.ws, run_info.run_name, "cancelled", {})        # , is_aml=self.is_aml)
@@ -1422,6 +1462,11 @@ class XTController(rpyc.Service):
             running_names = [run.run_name for run in self.runs.values() if run.status == "running"]
         return running_names
 
+    def get_alive_names(self):
+        with runs_lock:
+            running_names = [run.run_name for run in self.runs.values() if run.status in ["running", "queued", "spawning"]]
+        return running_names
+
     # def get_active_runs_count(self):
     #     ''' return runs that are:
     #         - queued
