@@ -10,6 +10,7 @@ import os
 import sys
 import math
 import time
+import yaml
 import random
 import argparse
 
@@ -25,8 +26,9 @@ import torchvision
 import PIL
 
 from xtlib import utils
-from xtlib import file_utils
 from xtlib import errors
+from xtlib import constants
+from xtlib import file_utils
 
 hvd = None     # imported on demand
 
@@ -58,6 +60,7 @@ def parse_cmdline_args():
     parser.add_argument('--tensorboard', type=int, default=1, metavar='N', help='if tensorboard logging is enabled')
     parser.add_argument('--xtlib', type=int, default=1, metavar='N', help='if xtlib usage is enabled')
     parser.add_argument('--env-vars', type=int, default=1, metavar='N', help='=1 to show name/value of all environment variables')
+    parser.add_argument('--tag-job', type=int, default=0, help='set =1 to tag associated with plotted_metrics=test-acc')
 
     # MINI MNIST
     parser.add_argument('--train-percent', type=float, default=0.001, metavar='TrainPercent', help='percent of training samples to use (default: .001)')
@@ -386,16 +389,19 @@ class Trainer():
             print("---> tb_path=", tb_path)
             self.run = XTRun(xt_logging=logging, aml_logging=logging, checkpoints_enabled=logging, tensorboard_path=tb_path)
 
+            #utils.debug_break()
+
+            if args.tag_job:
+                self.run.tag_job( {"plotted_metric": "test_acc"} )
+
             # if "call search API" test was specified and if we are running under XT
             if args.search_api and run.run_name:
                 fn_sweeps = os.path.join(file_utils.get_my_file_dir(__file__), "miniSweeps.yaml")
                 sweeps = file_utils.load_yaml(fn_sweeps)
-                hp_space_dict = sweeps["hparams"]
+                hp_space_dict = sweeps[constants.HPARAMS_DIR]
                 print("hp_space_dict=", hp_space_dict)
                 search_type = "random"
 
-                #utils.debug_break()
-
                 hp_set = run.get_next_hp_set_in_search(hp_space_dict, search_type=search_type)
                 print("hp_set=", hp_set)
 
@@ -577,13 +583,34 @@ class Trainer():
 
         return model, device, mnt_output_dir, local_output_dir
 
+    def apply_runset_file(self, args, fn):
+        #utils.debug_break()
+
+        fn = os.path.abspath(fn)
+        with open(fn, "rt") as infile:
+            yd = yaml.safe_load(infile)  
+
+        if not constants.HPARAM_RUNSET in yd:
+            errors.internal_error("found runset file without {} property: {}".format(constants.HPARAM_RUNSET, fn))
+
+        print("applying runset file to args: {}".format(fn))
+        
+        hd = yd[constants.HPARAM_RUNSET ]
+        
+        for prop, val in hd.items():
+            prop = prop.replace("-", "_")
+            setattr(args, prop, val)
+
     def run(self):
 
         print("args=", sys.argv)
         self.args = parse_cmdline_args()
-
         args = self.args
 
+        fn_runset = "runset.yaml"
+        if os.path.exists(fn_runset):
+            self.apply_runset_file(args, fn_runset)
+
         model, device, mnt_output_dir, local_output_dir = self.init_stuff()
 
         start_epoch = 1
@@ -658,7 +685,7 @@ class Trainer():
                     checkpoint_freq = float(args.checkpoint)
                     checkpoint_units = "epochs"
 
-        model_dir = os.getenv("XT_MODEL_DIR", "models/minimnist")
+        model_dir = os.getenv("XT_MODEL_DIR", "models/miniMnist")
         fn_model = model_dir + "/mnist_cnn.pt"
         self.fn_text_log = mnt_output_dir + "/text_log.txt"
 
@@ -701,6 +728,8 @@ class Trainer():
 
         if run:
             # ensure we close all logging
+            print("miniMNST: mongo call stats")
+            run.store.mongo.print_call_stats()
             run.close()
 
 def main():
