@@ -285,8 +285,11 @@ def make_tmp_dir(prefix, fixed_name=True):
     # :param:
     if fixed_name:
         tmp_dir = os.path.expanduser("~/.xt/tmp/" + prefix)
-        ensure_dir_deleted(tmp_dir)
-        os.makedirs(tmp_dir)
+
+        # we cannot always delete this directory, so don't try
+        #ensure_dir_deleted(tmp_dir)
+        #os.makedirs(tmp_dir)
+        ensure_dir_exists(tmp_dir)
     else:
         tmp_dir = tempfile.mkdtemp(prefix=prefix)
 
@@ -357,7 +360,7 @@ def root_name(fn):
 
 def load_yaml(fn): 
     with open(fn, "rt") as file:
-        data = yaml.load(file, Loader=yaml.FullLoader)
+        data = yaml.safe_load(file)  # , Loader=yaml.FullLoader)
     return data
 
 def save_yaml(yd, fn): 
