@@ -7,6 +7,7 @@ import os
 import sys
 import json
 import time
+import yaml
 import importlib
 import numpy as np
 
@@ -36,23 +37,25 @@ class HParamSearch():
         sweep_text = None
 
         if context.hp_config:
-        # read hp config file from experiment or job
+            # read hp config file from experiment or job
             if context.aggregate_dest == "experiment":
                 text = store.read_experiment_file(context.ws, context.dest_name, context.hp_config)
-            else:    # assume sweeps file is at the job level
+            else:    
+                # assume sweeps file is at the job level
                 text = store.read_job_file(context.dest_name, context.hp_config)
 
-            sweep_text = self.generate_hparam_set(context.hp_config, text, child_name, cmd_parts, store, context)
-            # update context (critical if this run gets preempted/restarted)
-            context.cmd_parts = cmd_parts
+            arg_dict = self.generate_hparam_set(context.hp_config, text, child_name, cmd_parts, store, context)
 
-        # elif context.using_hp:
-        #     # get a pre-generated set of hp values from the list
-        #     cmd_parts = self.get_cmd_from_sweeps_list(store, context, parent)
-        #     # update context (critical if this run gets preempted/restarted)
-        #     context.cmd_parts = cmd_parts
+            if context.option_prefix != None:
+                self.apply_runset_to_cmd_parts(arg_dict, cmd_parts, context)
+
+                # update context (critical if this run gets preempted/restarted)
+                context.cmd_parts = cmd_parts
+
+            # always write a YAML runset file
+            runset = {constants.HPARAM_RUNSET: arg_dict}
+            sweep_text = yaml.dump(runset)
 
-        if sweep_text:
             fn_target = context.fn_generated_config
             console.print("HP fn_target=", fn_target)
 
@@ -77,8 +80,8 @@ class HParamSearch():
 
         # we only support yaml format here
         import yaml
-        hparams = yaml.load(text)
-        hparams = hparams["hparams"]
+        hparams = yaml.safe_load(text)
+        hparams = hparams[constants.HPARAM_DIST]
 
         # algorithm-specific processing
         search_type = context.search_type
@@ -86,9 +89,7 @@ class HParamSearch():
 
         # get code_path for search_type from hpsearch_providers
         arg_dict = self.hp_search_core(context, search_type, store, run_name, records)
-
-        text = self.apply_hparam_set(arg_dict, cmd_parts, context)
-        return text
+        return arg_dict
 
     def hp_search_core(self, context, search_type, store, run_name, space_records):
         # get code_path for search_type from hpsearch_providers
@@ -106,10 +107,12 @@ class HParamSearch():
 
         arg_dict = impl.search(run_name, store, context, space_records, runs=runs)
 
-        # if not testing, output some debug info
-        if search_type != "random":
-            console.print("\n---- search_type={}, returned arg_dict={} ----\n".format(search_type, arg_dict))
+        # fix up returned values 
+        for key, value in arg_dict.items():
+            if isinstance(value, np.float64):
+                arg_dict[key] = value.item()
 
+        console.print("\n---- search_type={}, returned arg_dict={} ----\n".format(search_type, arg_dict))
         return arg_dict
 
     def parse_hp_config_yaml(self, hparams, search_type):
@@ -192,27 +195,15 @@ class HParamSearch():
 
         return runs
    
-    def apply_hparam_set(self, arg_dict, cmd_parts, context):
+    def apply_runset_to_cmd_parts(self, arg_dict, cmd_parts, context):
         # we would like to insert the new HP values BEFORE the potential override options
         # that are specified in cmd_parts, so try to set insert_index based on what we 
         # can figure out about cmd_parts
         insert_index = self.find_insert_index(cmd_parts)
 
-        new_lines = []
-
         for prop, value in arg_dict.items():
             # update cmd
-            if context.option_prefix != None:
-                insert_index = self.update_cmd(cmd_parts, insert_index, context.option_prefix, prop, value)
-
-            # update new config file text
-            new_line = prop + " = " + str(value)
-            new_lines.append(new_line)
-        
-        text = "\n".join(new_lines) + "\n"
-        console.print("new sweep text=\n", text)
-
-        return text
+            insert_index = self.update_cmd(cmd_parts, insert_index, context.option_prefix, prop, value)
 
     def find_insert_index(self, cmd_parts):
         console.print("find_insert_index: cmd_parts=", cmd_parts)
