@@ -239,7 +239,7 @@ class ImplCompute(ImplBase):
 
                         entries = backend.get_service_queue_entries(service_node_info)
                         if entries is None:
-                            console.print("  <not supported for this service")
+                            console.print("  <not supported for this service>")
                         else:
                             if entries:
                                 for entry in entries:
@@ -497,6 +497,16 @@ class ImplCompute(ImplBase):
                     else:
                         console.print("  {} has already exited, status={}".format(full_name, status))
 
+    def print_cancel_all_results(self, cancel_results_by_boxes):
+        for target, results in cancel_results_by_boxes.items():
+            console.print("Target: {}".format(target))
+            for result in results:
+                console.print(
+                    "canceled: {}, service_status: {}, simple_status: {}".format(
+                        result.get("cancelled"),
+                        result.get("service_status"),
+                        result.get("simple_status")))
+
     def get_runs_by_box(self, run_names, workspace=None):
         run_names, actual_ws = run_helper.parse_run_list(self.store, workspace, run_names)
 
@@ -544,7 +554,7 @@ class ImplCompute(ImplBase):
 
             cancel_results_by_box[box_name] = cancel_results
 
-        self.print_cancel_results(cancel_results_by_box)
+        self.print_cancel_all_results(cancel_results_by_box)
 
     #---- CANCEL JOB command ----
     @argument("job-id", help="the name of the job to cancel")
@@ -592,7 +602,7 @@ class ImplCompute(ImplBase):
             cancel_results = backend.cancel_runs_by_user(None)
             cancel_results_by_box[target] = cancel_results
 
-        self.print_cancel_results(cancel_results_by_box)
+        self.print_cancel_all_results(cancel_results_by_box)
 
     #---- RESTART CONTROLLER command ----
     @argument("job-id", help="the name of the job whose node will be restarted")
