@@ -38,7 +38,6 @@ from xtlib.helpers.feedbackProgress import FeedbackProgress
 from xtlib.qfe import command, argument, keyword_arg, option
 from xtlib.qfe import flag, faq, root, example, clone, command_help, see_also
 
-
 '''
 This module implements the following commands:
 
@@ -136,7 +135,7 @@ class ImplStorage(ImplBase):
     #---- CREATE SHARE command ----
     @argument("share", help="the name for the newly created share")
     @example(task="create a new share named 'trajectories", text="xt create share trajectories")
-    @command(kwgroup="create", kwhelp="create the specified storage object", help="creates a new XT share")
+    @command(kwgroup="create", help="creates a new XT share")
     def create_share(self, share):
         self.store.create_share(share)
         console.print("share created: " + share)
@@ -407,7 +406,7 @@ class ImplStorage(ImplBase):
 
                 if export:
                     sep_char = "\t"
-                    count = lb.export_records(export, ms["records"], ms["keys"], ms["keys"], sep_char)
+                    count = lb.export_records(export, ms["records"], ms["keys"], sep_char)
                     console.print("report exported to: {} ({} rows)".format(export, count-1))
                 else:
                     if not just_one:
@@ -501,18 +500,18 @@ class ImplStorage(ImplBase):
         pb.build()
 
     #---- EXTRACT command ----
-    @argument(name="run-name", help="the name of the run")
+    @argument(name="runs", type="str_list", help="a comma separated list of runs, jobs, or experiments", required=True)
     @argument(name="dest-dir", help="the path of the directory")
     @flag(name="browse", help="specifies that an folder window should be opened for the dest_dir after the extraction has completed")
     @option(name="workspace", default="$general.workspace", help="the workspace that the runs resides in")
     @option("response", default=None, help="the response to be used to confirm the directory deletion")
     @example(task="extract files from curious/run26 to c:\\run26_files", text="xt extract curious/run26 c:\\run26_files")
     @command(help="download all files associated with the run to the specified directory")
-    def extract(self, run_name, dest_dir, workspace, response, browse):
+    def extract(self, runs, dest_dir, workspace, response, browse):
         is_aml = self.is_aml_ws(workspace)
-        ws, run_name, full_run_name = run_helper.validate_run_name(self.store, workspace, run_name, parse_only=is_aml)
+        #ws, run_name, full_run_name = run_helper.validate_run_name(self.store, workspace, run_name, parse_only=is_aml)
 
-        console.print("extracting files for: {}...".format(run_name))
+        console.print("extracting files for: {}...".format(runs))
 
         extract = True
 
@@ -523,10 +522,47 @@ class ImplStorage(ImplBase):
 
         if extract:
             file_utils.ensure_dir_clean(dest_dir)
+
+            # first, determine nodes to be added to dest_dir
+            nodes = "run"   # default
+
+            for name_entry in runs:
+                if name_entry.startswith("job"):
+                    # if we find at least 1 job name specified, output to dest_dir/jobNNN/runNNN
+                    if not nodes == "exper":
+                        nodes = "job"
+                elif not name_entry.startswith("run"):
+                    # if we find at least 1 experiment, output to dest_dir/experFoo/jobNNN/runNNN
+                    # this takes priority over all other nodes values
+                    nodes = "exper"
             
-            # download files for run (and unzip as needed)
-            files = capture.download_run(self.store, ws, run_name, dest_dir)
-            console.print("{} files downloaded to: {}".format(len(files), dest_dir))
+            # convert list of runs, jobs, experiments into a list of runs
+            pure_run_list, actual_ws = run_helper.expand_run_list(self.store, self.store.mongo, workspace, runs)
+
+            for run_name in pure_run_list:
+
+                if nodes == "exper":
+                    # get exper_name and job_id of run
+                    record = self.store.mongo.get_info_for_runs(actual_ws, {"_id": run_name}, {"job_id": 1, "exper_name": 1})
+                    job_id = utils.safe_cursor_value(record, "job_id")
+                    exper_name = utils.safe_cursor_value(record, "exper_name")
+                    
+                    actual_dest_dir = "{}/{}/{}/{}".format(dest_dir, exper_name, job_id, run_name)
+
+                elif nodes == "job":
+                    # get job_id of run
+                    record = self.store.mongo.get_info_for_runs(actual_ws, {"_id": run_name}, {"job_id": 1})
+                    job_id = utils.safe_cursor_value(record, "job_id")
+
+                    actual_dest_dir = "{}/{}/{}".format(dest_dir, job_id, run_name)
+
+                else:
+                    # run_name only
+                    actual_dest_dir = "{}/{}".format(dest_dir, run_name)
+                    
+                # download files for run (and unzip as needed)
+                files = capture.download_run(self.store, actual_ws, run_name, actual_dest_dir)
+                console.print("  {} files downloaded to: {}".format(len(files), actual_dest_dir))
 
             if browse:
                 if pc_utils.is_windows():
@@ -583,20 +619,47 @@ class ImplStorage(ImplBase):
             if not filenames:
                 errors.store_error("Missing hp-config file for experiment={}".format(dest_name))
 
-        fn_hp_config = constants.HP_CONFIG_DIR + "/" + filenames[0]
+        for f in filenames:
+            if f.endswith('.yaml'):
+                filename = f
+
+        fn_hp_config = constants.HP_CONFIG_DIR + "/" + filename
         #console.print("found hp-config file: ", fn_hp_config)
 
+        # plotted_metric is the Y axis for big HX plot
+        plotted_metric = primary_metric    # default value
+
         if dest_name.startswith("job"):
             job_ws = self.store.get_job_workspace(dest_name)
             if job_ws:
-                workspace = job_ws
-                console.diag("{} found in ws={}".format(dest_name, workspace))
+                console.diag("{} found in ws={}".format(dest_name, job_ws))
             console.diag("after get_job_workspace call (mongo-db)")
 
-        # ws, hx_metric, score_rollup, cache_dir, aggregate_dest, dest_name, fn_hp_config
-        # , steps_name=steps_name, step_name=step_name, time_name=time_name, sr_name=sr_name, se_name=se_name)
-        hx = HyperparameterExplorer(self.store, workspace, primary_metric, cache_dir, aggregate_dest, 
-            dest_name, fn_hp_config, steps_name, log_interval_name, time_name, step_name, success_rate_name, sample_efficiency_name)
+            # See if this job has tags for metric names needed by HX.
+            filter_dict = {"job_id": dest_name}
+            fields_dict = {"tags.plotted_metric": 1, "tags.primary_metric": 1, "tags.step_name": 1}
+            records = self.store.mongo.get_info_for_jobs(filter_dict, fields_dict)
+            tags = utils.safe_cursor_value(records, "tags")
+            value = utils.safe_value(tags, "plotted_metric")
+            if value:
+                plotted_metric = value
+            value = utils.safe_value(tags, "primary_metric")
+            if value:
+                primary_metric = value
+            value = utils.safe_value(tags, "step_name")
+            if value:
+                step_name = value
+
+        hx = HyperparameterExplorer(
+            store=self.store,
+            ws_name=workspace,
+            run_group_type=dest_name,
+            run_group_name=aggregate_dest,
+            hp_config_cloud_path=fn_hp_config,
+            hp_config_local_dir=cache_dir,
+            plot_x_metric_name=step_name,
+            plot_y_metric_name=plotted_metric,
+            hist_x_metric_name=primary_metric)
         hx.run(timeout)
 
     #---- VIEW BLOB command ----
@@ -746,8 +809,7 @@ class ImplStorage(ImplBase):
     @example("xt list runs --filter='epochs > 100'", task="only display runs that have run for more than 100 epochs")
     @faq("how can I find out which columns are available", "use the --available flag")
     @faq("why do some runs show their status as 'created', even though they have completed", "runs that are executed in direct mode, on a service backend without using the XT controller, do not always update the XT database correctly")
-    @see_also("XT Filters", "filters")
-    @see_also("XT Columns", "columns")
+    @see_also("Using the XT CLI", "cmd_options")
     @command(kwgroup="list", pass_by_args=True, help="displays a run report for the specified runs")
     def list_runs(self, args):
         '''
@@ -799,8 +861,7 @@ class ImplStorage(ImplBase):
 
     # examples, FAQs
     @example(task="display a report of the last 5 jobs that were run", text="xt list jobs --last=5")
-    @see_also("XT Filters", "filters")
-    @see_also("XT Columns", "columns")
+    @see_also("Using the XT CLI", "cmd_options")
     @command(kwgroup="list", pass_by_args=True, help="displays a job report for the specified jobs")
     def list_jobs(self, args):
         '''
@@ -1065,10 +1126,14 @@ class ImplStorage(ImplBase):
     @option(name="experiment", help="the experiment that the path is relative to")
     @option(name="run", help="the run name that the path is relative to")
     @option(name="subdirs", type=int, help="controls the depth of subdirectories listed (-1 for unlimited)")
+    @flag(name="recursive", help="show all subfolders and their contents, recursively")
     @example(task="list blobs from store for job2998", text="xt list blobs --job=job2998")
     @example(task="list blobs from 'models' share", text="xt list blobs --share=models")
     @command(kwgroup="list", kwhelp="displays the specified storage items", help="lists the Azure store blobs matching the specified path/wildcard and options")
-    def list_blobs(self, path, share, workspace, experiment, job, run, subdirs):
+    def list_blobs(self, path, share, workspace, experiment, job, run, subdirs, recursive):
+
+        if recursive:
+            subdirs = -1
 
         if subdirs is None:
             subdirs = 0
@@ -1217,7 +1282,7 @@ class ImplStorage(ImplBase):
             fs.delete_file(rel_bn)
             console.print("removed")
 
-    def dump_mongo_doc(self, name, doc, prev_indent):
+    def dump_mongo_doc(self, name, doc, prev_indent, show_log_records=False):
         console.print("{}{}:".format(prev_indent, name))
         indent = prev_indent + "  "
         keys = list(doc.keys())
@@ -1225,16 +1290,33 @@ class ImplStorage(ImplBase):
 
         for key in keys:
             value = doc[key]
-            if isinstance(value, dict):
+
+            if key == "log_records":
+                if show_log_records:
+                    console.print("{}{}: {} records".format(indent, key, len(value)))
+                    for i, lr in enumerate(value):
+                        self.dump_mongo_doc("[{}]".format(i), lr, indent)
+                else:
+                    console.print("{}{}: {} records (not shown)".format(indent, key, len(value)))
+
+            elif isinstance(value, dict):
                 self.dump_mongo_doc(key, value, indent)
             else:
                 console.print("{}{}: {}".format(indent, key, value))
 
+    def load_template(self, name):
+        fn = file_utils.get_xtlib_dir() + "/templates/" + name
+        td = utils.load_json_file(fn)
+        return td
 
-    #---- CREATE TEAM command ----
+    #---- CREATE SERVICES TEMPLATE command ----
     @example(task="create a template for a new XT team", text="xt create services template")
-    @command(kwgroup="create", help="generate an Azure template for creating a set of resources for an XT Team")
-    def create_services_template(self):
+    @flag(name="base", help="generate a template to create XT base services")
+    @flag(name="batch", help="generate a template to create XT base services with Azure Batch")
+    @flag(name="all", help="generate a template to create all XT services")
+    @flag(name="aml", help="generate a template to create XT base services with Azure Machine Learning")
+    @command(kwgroup="create", kwhelp="create the specified storage object", help="generate an Azure template for creating a set of resources for an XT Team")
+    def create_services_template(self, base, batch, all, aml):
         '''Once you have run this command to generate a team template, follow these instructions to complete the process:
         
         1. browse to the Azure Portal Custom Template Deployment page: https://ms.portal.azure.com/#create/Microsoft.Template
@@ -1248,23 +1330,39 @@ class ImplStorage(ImplBase):
         9. after 5-15 minutes, you should receive a 'Deployment succeeded' message in the Azure Portal
         '''
 
-        # read generic template
-        fn_template = file_utils.get_xtlib_dir() + "/templates/teamResources.json"
-        template = file_utils.read_text_file(fn_template)
+        # if not flags specified, use BATCH as the default
+        if not (base or batch or aml or all):
+            batch = True
+
+        # always read base part of template
+        template = self.load_template("teamResourcesBase.json")
+
+        if batch or all:
+            bt = self.load_template("teamResourcesBatch.json")
+            template["resources"].extend(bt)
+
+        if aml or all:
+            at = self.load_template("teamResourcesAml.json")
+            template["resources"].extend(at)
+
+        # load the template as a string
+        template_text = json.dumps(template, indent=4)
 
         # # personalize it for team name
         # template = template.replace("teamx7", name)
+
+        # add user's object_id from azure active directory
+        object_id = self.config.get_vault_key("object_id")
+        template_text = template_text.replace("$object_id", object_id)
         
         # write to local directory
-        fn_team_template = "create_xt_team.json"
-        file_utils.write_text_file(fn_team_template, template)
+        fn_team_template = "azure_template.json"
+        file_utils.write_text_file(fn_team_template, template_text)
 
         # explain how to use
         PORTAL_URL = "https://ms.portal.azure.com/"
         TEMPLATE_URL = "https://ms.portal.azure.com/#blade/Microsoft_Azure_Marketplace/MarketplaceOffersBlade/selectedMenuItemId/home/searchQuery/template"
 
-        console.print()
-        console.print("template generated: {}".format(fn_team_template))
         console.print()
         console.print("To create the resources for your XT team:")
         console.print("  1. browse to the Azure Portal Custom Template Deployment page: https://ms.portal.azure.com/#create/Microsoft.Template")
@@ -1278,16 +1376,20 @@ class ImplStorage(ImplBase):
         console.print("  9. after 5-15 minutes, you should receive a 'Deployment succeeded' message in the Azure Portal")
         console.print("  10. at this point, you can create a new local XT config file for your team, for example:")
 
+        console.print()
+        console.print("--> template file generated: {}".format(fn_team_template))
+        console.print()
+
         # TODO: add --team option to xt config cmd
         #console.print("  > xt config --template=batch --team=YourTeamNameHere")
         
-
     #---- VIEW MONGO command ----
     @argument(name="name", help="the name of the run or job to show the mongo-db data for")
     @option(name="workspace", default="$general.workspace", help="the workspace that the run is defined in")
+    @flag(name="log-records", help="display set of log records for run")
     @example(task="view the mongo-db information for run23 in the curious workspace", text="xt view mongo curious/run23")
     @command(kwgroup="view", help="view the mongo-db JSON data associated with the specified run")
-    def view_mongo(self, name, workspace):
+    def view_mongo(self, name, workspace, log_records):
         # view MONGO-DB document for run or job
 
         if name.startswith("run"):
@@ -1298,11 +1400,12 @@ class ImplStorage(ImplBase):
             filter_dict = {}
 
             filter_dict["run_name"] = {"$in": run_names}
-            mongo_docs = self.store.get_ws_runs(ws, filter_dict, include_log)
+            mongo_docs = self.store.get_ws_runs(ws, filter_dict, True)
 
             for doc in mongo_docs:
                 #console.print("doc:")
-                self.dump_mongo_doc("\nMONGO-DB for " + full_run_name, doc, "")
+                self.dump_mongo_doc("\nMONGO-DB for " + full_run_name, doc, "", log_records)
+
         elif name.startswith("job"):
             ws = job_helper.validate_job_name_with_ws(self.store, name, True)
 
