@@ -11,10 +11,12 @@ import time
 import shutil
 import logging
 
+from xtlib import qfe
 from xtlib.console import console
 from xtlib.cmd_core import CmdCore
 from xtlib.helpers import xt_config
 from xtlib.impl_base import ImplBase
+from xtlib.helpers import file_helper
 from xtlib.backends.backend_philly import Philly
 from xtlib.helpers.hexdump import hex_dump
 from xtlib.qfe import inner_dispatch, see_also
@@ -77,20 +79,40 @@ class ImplUtilities(ImplBase):
     @option("console", type=int, values=["none", "normal", "diagnostics", "detail"], default="$internal.console", help="sets the level of console output")
     @flag("stack-trace", default="$internal.stack-trace", help="Show stack trace when an exception is raised")
     @flag("quick-start", default="$general.quick-start", help="When true, XT startup time is reduced (experimental)")
+    @flag("new", help="specifies that the current XT command should be run in a new console window")
+    @flag("echo", help="echo the XT command before running it (used with --new)")
     #@flag("prep", hidden=True, help="Prepares local machine for running the XT controller")
     @root(help="callback for processing root flag detection")
     def root(self, name, value):
         #console.print("setting root name={} to value={}".format(name, value))
         if name == "help":
             pass       # will be handled later
+
         elif name == "console":
             console.set_level(value)
+
         elif name == "stack-trace":
             utils.show_stack_trace = value
+
+        elif name == "new":
+            if value and process_utils.can_create_console_window():
+                cmd = qfe.current_dispatcher.dispatch_cmd
+                echo_cmd = "xt " + cmd.replace("--new", "--echo", 1)
+
+                process_utils.run_cmd_in_new_console(echo_cmd)
+                errors.early_exit_without_error()
+
+        elif name == "echo":
+            if value:
+                cmd = qfe.current_dispatcher.dispatch_cmd
+                console.print("xt " + cmd, flush=True)
+
         elif name == "quick-start":
             pass       # was already handled
+
         elif name == "prep":
             self.prep_machine_for_controller()
+            
         else:
             errors.syntax_error("unrecognized root flag=" + name)
 
@@ -124,14 +146,50 @@ class ImplUtilities(ImplBase):
         if status:
             console.print("public key successfully sent.")
 
+    #---- ZIP command ----
+    @argument(name="files", help="a directory or wildcard string identifying the files to be zipped")
+    @argument(name="zipfile", help="name of the zip file to be created")
+    @example(task="zip up all the files in the test directory to test.zip", text="xt zip test test.zip")
+    @command(help="compresses the specified files and writes them to the zip file")
+    def zip(self, files, zipfile):
+        filenames = file_helper.get_filenames_from_include_lists([files], [".git", "__pycache__"], recursive=True)
+        count = len(filenames)
+        source_dir = os.path.dirname(files)
+        remove_prefix_len = 1 + len(source_dir)
+
+        file_helper.zip_up_filenames(zipfile, filenames, True, remove_prefix_len)
+        console.print("{:,} files written to: {}".format(count, zipfile))
+
+    #---- UNZIP command ----
+    @argument(name="zipfile", help="name of the zip file to be read")
+    @argument(name="destination", help="the directory where the files should be unzipped")
+    @example(task="unzip all files from test.zip to the 'test' direcotry", text="xt unzip test.zip test")
+    @command(help="uncompress all of the files in the specified zip file to the destination directory")
+    def unzip(self, zipfile, destination):
+        names = file_helper.unzip_files(zipfile, destination)
+        console.print("{:,} files extacted to: {}".format(len(names), destination))
+
+    #---- WGET command ----
+    @argument(name="url", help="URL of the file to be downloaded")
+    @argument(name="fn-output", help="name of the local file to be created")
+    @example(task="display the text for http://google.com", text="xt wget http://google.com goggle.txt")
+    @command(help="download the specified file from a web URL")
+    def wget(self, url, fn_output):
+        import urllib.request
+        
+        console.print("downloading file from: {} ...".format(url))
+        urllib.request.urlretrieve(url, fn_output)
+        console.print("downloaded to: {}".format(fn_output))
+
     #---- SSH command ----
     @argument(name="name", help="the box or run name to communicate with or connect to")
     @argument(name="cmd", help="the optional command to execute", required=False, type="text", default="")
     @option(name="workspace", default="$general.workspace", help="the workspace for the runs to be displayed")
+    @option(name="output", help="the name of the file to write the cmd output to")
     @example(task="initiate a remote console session with box 'vm10'", text="xt ssh vm10")
     @example(task="get a directory listing of files on box 'vm23''", text="xt ssh vm10 ls -l")
-    @command(help="executes the specified command, on begins a console session, with the specified box")
-    def ssh(self, name, cmd, workspace):
+    @command(options_before_args=True, help="executes the specified command, on begins a console session, with the specified box")
+    def ssh(self, name, cmd, workspace, output):
 
         capture_output = True if cmd else False
 
@@ -152,10 +210,17 @@ class ImplUtilities(ImplBase):
             ssh_ip = info["box_addr"]
 
             #console.print("ssh_cmd: ssh_ip=", ssh_ip, ", cmd=", cmd)
-            exit_code, output = process_utils.sync_run_ssh(self, ssh_ip, cmd, capture_output=capture_output)
+            capture_as_bytes = bool(output)
 
-        if capture_output:
-            console.print(output)
+            exit_code, ssh_output = process_utils.sync_run_ssh(self, ssh_ip, cmd, capture_output=capture_output,
+                capture_as_bytes=capture_as_bytes)
+
+        if output:
+            # write as bytes
+            with open(output, "wb") as outfile:
+                outfile.write(ssh_output)
+        elif capture_output:
+            console.print(ssh_output)
 
     #---- SCP command ----
     @argument(name="cmd", help="the scp command to execute", type="text")
@@ -188,10 +253,13 @@ class ImplUtilities(ImplBase):
                         #console.print("new part=", new_part)
                         parts[i] = new_part
 
-        cmd = " ".join(parts).replace(": ", ":")
+        #cmd = " ".join(parts).replace(": ", ":")
         #console.print("new cmd=", cmd)
 
-        exit_code, output = process_utils.run_scp_cmd(self, cmd, report_error=True)
+        # remove empty parts
+        parts = [part for part in parts if part]
+
+        exit_code, output = process_utils.run_scp_cmd(self, parts, report_error=True)
         if output:
             console.print(output)
         else:
