@@ -213,22 +213,28 @@ def list_jobs(store, config, args):
             console.print(line)
     else:            
         avail_list = list(user_to_actual.keys())
-        lines, row_count = builder.build_report(records, report_type="job-reports", args=args)
+        lines, row_count, was_exported = builder.build_report(records, report_type="job-reports", args=args)
 
-        # console.print the report
-        if row_count > 0:
+        if was_exported:
             console.print("")
 
             for line in lines:
                 console.print(line)
-
-            if row_count > 1:
-                if using_default_last:
-                    console.print("total jobs listed: {} (defaulted to --last={})".format(row_count, last))
-                else:
-                    console.print("total jobs listed: {}".format(row_count))
         else:
-            console.print("no matching jobs found")
+            # console.print the report
+            if row_count > 0:
+                console.print("")
+
+                for line in lines:
+                    console.print(line)
+
+                if row_count > 1:
+                    if using_default_last:
+                        console.print("total jobs listed: {} (defaulted to --last={})".format(row_count, last))
+                    else:
+                        console.print("total jobs listed: {}".format(row_count))
+            else:
+                console.print("no matching jobs found")
 
 
 def extract_tag_cols(records):
