@@ -980,82 +980,6 @@ class Dispatcher():
 
         return ad
         
-    def process_raw_options(self, options, collection, arg_dict):
-        # remove the hidden options for parsing a command
-        options = [opt for opt in options if not utils.safe_value(opt, "hidden")]
-
-        for item in collection:
-            undashed_name = item["name"]
-            value = item["value"]
-
-            opt_info = self.list_item_by_name(undashed_name, options)
-            if not opt_info:
-                self.syntax_error("cmd '{}', unrecognized option: --{}".format(self.cmd_words, undashed_name))
-
-            opt_name = opt_info["name"]
-            opt_type = opt_info["type"]
-            required = utils.safe_value(opt_info, "required")
-
-            multiple = opt_info["multiple"] if "multiple" in opt_info else None
-            values = opt_info["values"] if "values" in opt_info else None
-            
-            # is this a flag?
-            if opt_type == "flag":
-                if value is None:
-                    value = True
-                else:
-                    value = self.parse_flag_value(opt_name, value)
-            elif value is None:
-                if required:
-                    self.syntax_error("command option missing value: " + opt_name)
-
-            value = self.process_option_value(opt_name, opt_type, value, values)
-
-            if multiple:
-                if not opt_name in arg_dict:
-                    arg_dict[opt_name] = []
-                values = arg_dict[opt_name]
-                values.append(value)
-            else:
-                arg_dict[opt_name] = value 
-
-            # global explict_options
-            # explict_options[opt_name] = value
-
-    def collect_raw_options(self, collection, scanner, tok):
-        '''
-        NOTE: this can be called from before the command or after the command arguments, so
-        we don't process the value of the options, or exand the option name until later, 
-        when we are sure we know the command in effect.
-        '''
-
-        while tok:
-            dash_name = tok
-            #console.print("after name scan  : tok=", tok)
-
-            if not dash_name.startswith("--"):
-                break 
-
-            tok = scanner.scan()    # skip over the options/flag name
-
-            # remove leading dashes
-            undashed_name = dash_name[2:]
-
-            if tok == "=":
-                # skip over optional equals sign
-                tok = scanner.scan()
-
-                #console.print("scanning value: tok=", tok)
-                value, tok = self.scan_raw_value(scanner, tok)
-            else:
-                # flag
-                value = None
-
-            # store raw value
-            collection.append( {"name": undashed_name, "value": value} )
-
-        return tok
-
     def get_dispatch_cmd(self):
         return self.dispatch_cmd
 
@@ -1417,7 +1341,10 @@ class Dispatcher():
                     value = False
                 else:
                     self.syntax_error("Illegal value for boolean option: " + str(value))
-            elif opt_type != "str":
+            elif opt_type == "str":
+                value = self.expand_system_values(value)
+
+            else:
                 errors.internal_error("Unsupported opt_type={}".format(opt_type))
         
         return value, tok
\ No newline at end of file
