@@ -389,6 +389,8 @@ class ReportBuilder():
         values in each record must have been flattened with a dotted key (e.g., hparams.lr).
         records must be in final sort order.
         '''
+        row_count = 0
+        was_exported = False
 
         avail_list, user_col_args  = self.get_actual_and_user_cols(records, args)
 
@@ -397,10 +399,15 @@ class ReportBuilder():
 
         fn_export = args["export"]
         if fn_export:
-            sep_char = "\t"
-            count = self.export_records(fn_export, records, avail_list, actual_cols, sep_char)
-            line = "report exported to: {} ({} rows)".format(fn_export, count-1)
+            fn_ext = os.path.splitext(fn_export)[1]
+            sep_char = "," if fn_ext == ".csv" else "\t"
+
+            col_list = user_col_args.keys()
+            count = self.export_records(fn_export, records, col_list, sep_char)
+            row_count = count - 1
+            line = "report exported to: {} ({} rows)".format(fn_export, row_count)
             lines = [line]
+            was_exported = True
         else:
             group_by = args["group"] if "group" in args else None
             number_groups = args["number_groups"] if "number_groups" in args else False
@@ -409,11 +416,10 @@ class ReportBuilder():
             text, row_count = self.build_formatted_table(records, avail_cols=avail_list, col_list=actual_cols, 
                 report_type=report_type, group_by=group_by, number_groups=number_groups)
             lines = text.split("\n")
-        return lines, row_count
 
-    def export_records(self, fn_report, records, avail_cols, col_list, sep_char):
-        if not col_list:
-            col_list = avail_cols
+        return lines, row_count, was_exported
+
+    def export_records(self, fn_report, records, col_list, sep_char):
 
         lines = []
 
@@ -493,6 +499,30 @@ class ReportBuilder():
         
         return value
 
+    def needed_precision(self, value, significance):
+        # start with scientific notation formatting
+        text = "%.*e" % (significance-1, value)
+
+        # how much precision is required for specified significance?
+        sd, exponent = text.split("e")
+        exponent = int(exponent)
+
+        if exponent < 0:
+            needed = 1 - exponent
+        else:
+            needed = 0
+
+        return needed
+
+    def smart_float_format(self, value, significance, max_precision, max_fixed_length):
+        if max_precision <= max_fixed_length:
+            # FIXED POINT formatting
+            text = "%.*f" % (max_precision, value)
+        else:
+            # SCIENTIFIC NOTATION formatting
+            text = "%.*e" % (significance-1, value)
+        return text
+
     def build_formatted_table(self, records, avail_cols, col_list=None, max_col_width=None, 
         report_type="run-reports", group_by=None, number_groups=False):
         '''
@@ -512,6 +542,9 @@ class ReportBuilder():
             max_col_width = int(self.config.get(report_type, "max-width"))    
             
         precision = self.config.get(report_type, "precision")
+        significance = self.config.get(report_type, "significance")
+        max_fixed_length = self.config.get(report_type, "max-fixed-length")
+        
         uppercase_hdr_cols = self.config.get(report_type, "uppercase-hdr")
         right_align_num_cols = self.config.get(report_type, "right-align-numeric")
         truncate_with_ellipses = self.config.get(report_type, "truncate-with-ellipses")
@@ -531,6 +564,11 @@ class ReportBuilder():
         # build COL_INFO for each col (will calcuate col WIDTH, formatting, etc.)
         for i, col in enumerate(col_list):
 
+            if col == "hparams.lr":
+                print()
+            # largest precision needed for this col
+            max_precision = precision 
+
             # examine all records for determining max col_widths
             if self.user_col_args:
                 user_args = self.user_col_args[col]
@@ -575,10 +613,16 @@ class ReportBuilder():
                     else:
                         value_str = user_fmt.format(value)
                 elif isinstance(value, float):
-                    value_str = float_fmt.format(value)
+
+                    # default (smart) FLOAT formatting
+                    needed = self.needed_precision(value, significance)
+                    max_precision = max(max_precision, needed)
+                    value_str = self.smart_float_format(value, significance, max_precision, max_fixed_length)
+                    
                     if value_type == str:
                         value_type = float
                         is_numeric = True
+
                 elif isinstance(value, bool):
                     value_str = str(value)
                     value_type = bool
@@ -606,11 +650,15 @@ class ReportBuilder():
                 #console.print("name=", record["name"], ", col=", col, ", value_str=", value_str, ", col_width=", col_width)
 
             # finish this col
-            if is_numeric and not precision:
-                precision = 3
+            if is_numeric and not max_precision:
+                max_precision = 3
 
             col_width = min(max_col_width, col_width)
-            col_info = {"name": col, "user_name": user_col, "col_width": col_width, "value_type": value_type, "is_numeric": is_numeric, "precision": precision, "user_fmt": user_fmt, "value_padding": None}
+
+            col_info = {"name": col, "user_name": user_col, "col_width": col_width, "value_type": value_type, 
+                "is_numeric": is_numeric, "precision": max_precision, "user_fmt": user_fmt, 
+                "value_padding": None}
+
             col_infos.append(col_info)
             #console.print(col_info)
 
@@ -629,7 +677,7 @@ class ReportBuilder():
                     text += "\n{}:\n".format(group)
 
                 txt, rc = self.generate_report(col_infos, grecords, right_align_num_cols, uppercase_hdr_cols, truncate_with_ellipses, 
-                    col_space, duration_col_names, time_col_names)
+                    col_space, duration_col_names, time_col_names, significance, max_fixed_length)
 
                 # indent report
                 txt = "  " + txt.replace("\n", "\n  ")
@@ -641,12 +689,12 @@ class ReportBuilder():
         else:
             # UNGROUPED REPORT
             text, row_count = self.generate_report(col_infos, records, right_align_num_cols, uppercase_hdr_cols, truncate_with_ellipses, 
-                col_space, duration_col_names, time_col_names)
+                col_space, duration_col_names, time_col_names, significance, max_fixed_length)
 
         return text, row_count
 
     def generate_report(self, col_infos, records, right_align_num_cols, uppercase_hdr_cols, truncate_with_ellipses, 
-        col_space, duration_col_names, time_col_names):
+        col_space, duration_col_names, time_col_names, significance, max_fixed_length):
 
         # process COLUMN HEADERS
         text = ""
@@ -734,9 +782,18 @@ class ReportBuilder():
                         value = "" if value is None else value
                         text += str_fmt.format(value)
                     elif isinstance(value, float):
+
+                        # default (smart) FLOAT formatting
                         precision = col_info["precision"] if "precision" in col_info else float_precision
-                        float_fmt = "{:" + align + str(col_width) + "." + str(precision) + "f}"
+                        if precision >= max_fixed_length:
+                            # use SCIENTIFIC NOTATION
+                            float_fmt = "{:" + align + str(col_width) + "." + str(significance-1) + "e}"
+                        else:
+                            # use FIXED POINT formatting
+                            float_fmt = "{:" + align + str(col_width) + "." + str(precision) + "f}"
+
                         text += float_fmt.format(value)
+
                     elif isinstance(value, bool):
                         bool_fmt = "{!r:" + align + str(col_width) + "}"
                         text += bool_fmt.format(value)
