@@ -12,11 +12,12 @@ from collections import OrderedDict
 
 # xtlib
 from xtlib import utils
+from xtlib import errors
 from xtlib import constants
 from xtlib import file_utils
 from xtlib import impl_storage
 
-from xtlib.storage.store import Store
+from xtlib.storage.store import Store, store_from_context
 from xtlib.console import console
 from xtlib.helpers.xt_config import get_merged_config
 
@@ -131,8 +132,11 @@ class Run():
                 
             self.config = config
 
-            self.store = Store(self.store_creds, provider_code_path=provider_code_path, run_cache_dir=run_cache_dir, 
-                mongo_conn_str=self.mongo_conn_str, config=config)
+            if self.context:
+                self.store = store_from_context(self.context)
+            else:
+                self.store = Store(self.store_creds, provider_code_path=provider_code_path, run_cache_dir=run_cache_dir, 
+                    mongo_conn_str=self.mongo_conn_str, config=config)
 
             # if not supress_normal_output:
             #     console.print("XT logging enabled: ", self.run_name)
@@ -439,4 +443,34 @@ class Run():
         space_records = hparam_search.parse_hp_config_yaml(hp_space_dict, search_type)
 
         hp_dict = hparam_search.hp_search_core(self.context, search_type, self.store, run_name=self.run_name, space_records=space_records)
-        return hp_dict
\ No newline at end of file
+        return hp_dict
+
+    def tag_run(self, tag_dict):
+        if not self.run_name:
+            errors.api_error("tag_run requires app to be run under XT")
+
+        # convert tag dict to tag list
+        tag_list = []
+        for key, value in tag_dict.items():
+            if value is None:
+                tag_list.append(key)
+            else:
+                tag_list.append("{}={}".format(key, value))
+
+        storage = impl_storage.ImplStorage(self.config, self.store)
+        storage.set_tags([self.run_name], tag_list, self.ws_name)
+
+    def tag_job(self, tag_dict):
+        if not self.context:
+            errors.api_error("tag_job requires app to be run under XT")
+
+        # convert tag dict to tag list
+        tag_list = []
+        for key, value in tag_dict.items():
+            if value is None:
+                tag_list.append(key)
+            else:
+                tag_list.append("{}={}".format(key, value))
+
+        storage = impl_storage.ImplStorage(self.config, self.store)
+        storage.set_tags([self.context.job_id], tag_list, self.ws_name)
