@@ -60,7 +60,7 @@ def expand_run_entry(store, mongo, workspace, run_list, name):
         actual_ws = workspace
     elif "*" in name:
         #match wildcard to all run names in workspace
-        re_pattern = utils.wildcard_to_regex(first_name)
+        re_pattern = utils.wildcard_to_regex(name)
         filter_dict = {"_id": {"$regex": re_pattern} }
 
         records = mongo.get_info_for_runs(workspace, None, {"_id": 1})
@@ -337,22 +337,28 @@ def list_runs(store, config, args):
             console.print(line)
     else:            
         #avail_list = list(user_to_actual.keys())
-        lines, row_count = builder.build_report(records, report_type="run-reports", args=args)
+        lines, row_count, was_exported = builder.build_report(records, report_type="run-reports", args=args)
 
-        # console.print the report
-        if row_count > 0:
+        if was_exported:
             console.print("")
 
             for line in lines:
                 console.print(line)
-
-            if row_count > 1:
-                if using_default_last:
-                    console.print("total runs listed: {} (defaulted to --last={})".format(row_count, last))
-                else:
-                    console.print("total runs listed: {}".format(row_count))
         else:
-            console.print("no matching runs found")
+            # console.print the report
+            if row_count > 0:
+                console.print("")
+
+                for line in lines:
+                    console.print(line)
+
+                if row_count > 1:
+                    if using_default_last:
+                        console.print("total runs listed: {} (defaulted to --last={})".format(row_count, last))
+                    else:
+                        console.print("total runs listed: {}".format(row_count))
+            else:
+                console.print("no matching runs found")
 
 def get_run_records(store, workspace, run_names, fields_dict=None):
     ''' return run records for specified run names'''
@@ -506,7 +512,7 @@ def validate_run_name(store, ws, run_name, error_if_invalid=True, parse_only=Fal
 
     run_name = run_name.lower()
     if not parse_only and not "*" in run_name:
-        if not store.does_run_exist(ws, run_name):
+        if not store.mongo.does_run_exist(ws, run_name):
             if error_if_invalid:
                 errors.store_error("run '{}' does not exist in workspace '{}'".format(run_name, ws))
             else:
