@@ -282,7 +282,6 @@ class Runner():
             - logs a "cmd" record in the run log
             - log an optional "notes" record in the run log
             - captures the run's "before" files to the store's run directory
-
         '''
         console.diag("create_run: start")
 
@@ -714,6 +713,9 @@ class Runner():
         # build box: runs dict for job info file
         runs_by_box, last_run = self.build_runs_by_box(job_runs, workspace)
 
+        node_count = len(boxes)
+        args["node_count"] = node_count
+
         # now that we have run names for all static run names for all nodes, we can adjust cmds (and before files) for using the controller
         if use_controller:
             # we will create 2 temp. controller files in the CURRENT DIRECTORY (that will be captured to JOB)
@@ -842,38 +844,38 @@ class Runner():
         # return values for API support (X)
         return cmds, run_specs, using_hp, using_aml_hparam, sweeps_text, pool_info, job_id 
 
-    def distribute_cmds_to_nodes(self, cmds, num_nodes):
-        cmds_by_node = {}
+    # def distribute_cmds_to_nodes(self, cmds, num_nodes):
+    #     cmds_by_node = {}
         
-        # set current node
-        node_index = 0
+    #     # set current node
+    #     node_index = 0
 
-        # build cmd_parts and distribute them among nodes
-        for cmd in cmds:
-            node_id = "node" + str(node_index)
+    #     # build cmd_parts and distribute them among nodes
+    #     for cmd in cmds:
+    #         node_id = "node" + str(node_index)
 
-            if not node_id in cmds_by_node:
-                cmds_by_node[node_id] = []
+    #         if not node_id in cmds_by_node:
+    #             cmds_by_node[node_id] = []
 
-            # split on unquoted spaces
-            cmd_parts = utils.cmd_split(cmd)
-            cmds_by_node[node_id].append(cmd_parts)
+    #         # split on unquoted spaces
+    #         cmd_parts = utils.cmd_split(cmd)
+    #         cmds_by_node[node_id].append(cmd_parts)
 
-            node_index += 1
+    #         node_index += 1
 
-            if node_index >= num_nodes:
-                node_index = 0
+    #         if node_index >= num_nodes:
+    #             node_index = 0
 
-        return cmds_by_node
+    #     return cmds_by_node
 
-    def copy_cmds_to_nodes(self, cmds, num_nodes):
-        cmds_by_node = {}
+    # def copy_cmds_to_nodes(self, cmds, num_nodes):
+    #     cmds_by_node = {}
 
-        for node_index in range(num_nodes):
-            node_id = "node" + str(node_index)
-            cmds_by_node[node_id] = cmds
+    #     for node_index in range(num_nodes):
+    #         node_id = "node" + str(node_index)
+    #         cmds_by_node[node_id] = cmds
 
-        return cmds_by_node
+    #     return cmds_by_node
 
     def fixup_script_in_cmd(self, cmd):
         cmd_parts = utils.cmd_split(cmd)
@@ -1055,7 +1057,7 @@ class Runner():
         option_prefix = args["option_prefix"]
         search_type = args["search_type"]
         fn_sweeps = args["hp_config"]
-        static_search = True        # hard code this, for now
+        static_search = bool(option_prefix)    # enabled only if we use cmdline options for HP runsets
 
         if not cmd_line:
             cmd_line = " ".join(cmd_parts)
