@@ -36,6 +36,7 @@ class MongoDB():
         self.run_cache_dir = run_cache_dir
         self.mongo_client = None
         self.mongo_db = None
+        self.call_stats = {}
 
         # keep a count of how many retryable errors we have encountered
         self.retry_errors = 0
@@ -45,6 +46,11 @@ class MongoDB():
         # initialize mondo-db now
         self.init_mongo_db_connection()
 
+        # controls for mongo stats and logging
+        self.update_job_stats = True
+        self.update_run_stats = True
+        self.add_log_records = True
+
     #---- UTILS ----
 
     def get_service_name(self):
@@ -168,18 +174,27 @@ class MongoDB():
     def get_next_end_id(self, ws_name, default_next_run=1):
         return self.get_next_sequential_ws_id(ws_name, "next_end", default_next_run)
 
-    def mongo_with_retries(self, name, mongo_cmd, ignore_error=False):
+    def mongo_with_retries(self, name, mongo_cmd, ignore_error=False, return_records=False):
+        import pymongo.errors
+
         retry_count = 25
         result = None
-        import pymongo.errors
+        started = time.time()
 
         for i in range(retry_count):
             try:
                 result = mongo_cmd()
+
+                # most of the time, we want to ALSO want to retry building a record set from the cursor
+                if return_records:
+                    result = list(result) if result else []
                 break
             # watch out for these exceptions: AutoReconnect, OperationFailure (and ???)
             except BaseException as ex:   # pymongo.errors.OperationFailure as ex:
                 
+                # this is to help us debug situations where we raise the exception without ever printing retry msgs
+                print("got exception in mongo, i={}, retry_count={}, caller={}".format(i, retry_count, name), flush=True)
+
                 # since we cannot config logger to supress stderr, don't log this
                 #logger.exception("Error in mongo_with_retries, ex={}".format(ex))
                 
@@ -209,8 +224,33 @@ class MongoDB():
                     
                 time.sleep(backoff)
                 
+        # track all mongo calls stats
+        elapsed = time.time() - started
+
+        if not name in self.call_stats:
+            self.call_stats[name] = []
+        self.call_stats[name].append(elapsed)
+
+        #print("--> mongo call: {} (elapsed: {:.4f} secs)".format(name, elapsed))
         return result
 
+    def print_call_stats(self):
+        total_count = 0
+        total_time = 0
+        total_calls = 0
+
+        for name, stats in self.call_stats.items():
+            mean = np.mean(stats)
+            print("  {}x {}: avg={:.4f}".format(len(stats), name, mean))
+
+            total_calls += len(stats)
+            total_time += np.sum(stats)
+            total_count += 1
+
+        print()
+        print("  {}x {}: total={:.4f} secs".format(total_calls, "CALLS", total_time))
+        print()
+
     #---- RUNS ----
 
     def create_mongo_run(self, dd):
@@ -240,33 +280,35 @@ class MongoDB():
 
     def add_run_event(self, ws_name, run_name, log_record):
 
-        # first, add log record to ws/run document
-        update_doc = { "$push": {"log_records": log_record} }
-        self.mongo_with_retries("add_run_event", lambda: self.mongo_db[ws_name].update_one( {"_id": run_name}, update_doc, upsert=True) )
+        if self.add_log_records:
+            # first, add log record to ws/run document
+            update_doc = { "$push": {"log_records": log_record} }
+            self.mongo_with_retries("add_run_event", lambda: self.mongo_db[ws_name].update_one( {"_id": run_name}, update_doc, upsert=True) )
 
-        event_name = log_record["event"]
-        data_dict = log_record["data"]
-        updates = {}
+        if self.update_run_stats:
+            event_name = log_record["event"]
+            data_dict = log_record["data"]
+            updates = {}
 
-        if event_name == "hparams":
-            # create a "hparams" dict property for the run record
-            self.flatten_dict_update(updates, "hparams", data_dict)
-            self.update_mongo_run_from_dict(ws_name, run_name, updates)
+            if event_name == "hparams":
+                # create a "hparams" dict property for the run record
+                self.flatten_dict_update(updates, "hparams", data_dict)
+                self.update_mongo_run_from_dict(ws_name, run_name, updates)
 
-        elif event_name == "metrics":
-            # create a "metrics" dict property for the run record (most recent metrics)
-            self.flatten_dict_update(updates, "metrics", data_dict)
-            self.update_mongo_run_from_dict(ws_name, run_name, updates)
+            elif event_name == "metrics":
+                # create a "metrics" dict property for the run record (most recent metrics)
+                self.flatten_dict_update(updates, "metrics", data_dict)
+                self.update_mongo_run_from_dict(ws_name, run_name, updates)
 
-        if event_name == "started":
-            updates = { "status": "running" }
-            #console.print("updating run STATUS=", updates)
-            self.update_mongo_run_from_dict(ws_name, run_name, updates)
+            if event_name == "started":
+                updates = { "status": "running" }
+                #console.print("updating run STATUS=", updates)
+                self.update_mongo_run_from_dict(ws_name, run_name, updates)
 
-        elif event_name == "status-change":
-            updates = { "status": data_dict["status"] }
-            #console.print("updating run STATUS=", updates)
-            self.update_mongo_run_from_dict(ws_name, run_name, updates)
+            elif event_name == "status-change":
+                updates = { "status": data_dict["status"] }
+                #console.print("updating run STATUS=", updates)
+                self.update_mongo_run_from_dict(ws_name, run_name, updates)
 
     def flatten_dict_update(self, updates, dd_name, dd):
         for key, value in dd.items():
@@ -274,51 +316,57 @@ class MongoDB():
 
     def update_mongo_run_at_end(self, ws_name, run_name, status, exit_code, restarts, end_time, log_records, hparams, metrics):
         # update run document on Mongo DB
-        #run_doc = self.mongo_db[ws_name].find_one( {"_id": run_name} )
-        # update properties
-        updates = {}
-        updates["status"] = status
-        updates["exit_code"] = exit_code
-        updates["restarts"] = restarts
-        updates["end_time"] = end_time
 
-        # add the unique end_id (relative to ws_name)
-        updates["end_id"] = self.get_next_end_id(ws_name)
+        if self.update_run_stats:
+            # update properties
+            updates = {}
+            updates["status"] = status
+            updates["exit_code"] = exit_code
+            updates["restarts"] = restarts
+            updates["end_time"] = end_time
 
-        # structured properties
-        if hparams:
-            #updates["hparams"] = hparams
-            self.flatten_dict_update(updates, "hparams", hparams)
+            # add the unique end_id (relative to ws_name)
+            updates["end_id"] = self.get_next_end_id(ws_name)
 
-        if metrics:
-            #updates["metrics"] = metrics
-            self.flatten_dict_update(updates, "metrics", metrics)
-        
-        # no longer need this step here (log records are now appended as they are logged)
-        #updates["log_records"] = log_records
+            # structured properties
+            if hparams:
+                #updates["hparams"] = hparams
+                self.flatten_dict_update(updates, "hparams", hparams)
 
-        self.update_mongo_run_from_dict(ws_name, run_name, updates)
+            if metrics:
+                #updates["metrics"] = metrics
+                self.flatten_dict_update(updates, "metrics", metrics)
+            
+            # no longer need this step here (log records are now appended as they are logged)
+            #updates["log_records"] = log_records
+
+            self.update_mongo_run_from_dict(ws_name, run_name, updates)
 
     def update_mongo_run_from_dict(self, ws_name, run_name, dd):
         #console.print("update_mongo_run_from_dict: ws_name={}, run_name={}, dd={}".format(ws_name, run_name, dd))
 
-        update_dd = copy.deepcopy(dd)
-        update_dd["last_time"] = utils.get_time()
+        if self.update_run_stats:
+            update_dd = copy.deepcopy(dd)
+            update_dd["last_time"] = utils.get_time()
 
-        update_doc = { "$set": update_dd}
+            update_doc = { "$set": update_dd}
 
-        #console.print("update_mongo_run_from_dict: ws_name={}, run_name={}, update_doc={}".format(ws_name, run_name, update_doc))
+            #console.print("update_mongo_run_from_dict: ws_name={}, run_name={}, update_doc={}".format(ws_name, run_name, update_doc))
 
-        # do a REPLACE DOC update operation
-        self.mongo_with_retries("update_mongo_run_from_dict", lambda: self.mongo_db[ws_name].update_one( {"_id": run_name}, update_doc) )
+            # do a REPLACE DOC update operation
+            self.mongo_with_retries("update_mongo_run_from_dict", lambda: self.mongo_db[ws_name].update_one( {"_id": run_name}, update_doc) )
+
+    def does_run_exist(self, ws, run_name):
+        records = self.get_info_for_runs(ws, {"_id": run_name}, {"_id": 1})
+        exists = len(records) == 1
+        return exists
 
     def get_info_for_runs(self, ws_name, filter_dict, fields_dict=None):
 
         # filter_dict = {}
         # filter_dict["run_name"] = {"$in": run_names}
 
-        cursor = self.mongo_with_retries("get_boxes_for_runs", lambda: self.mongo_db[ws_name].find(filter_dict, fields_dict))
-        run_records = list(cursor) if cursor else []
+        run_records = self.mongo_with_retries("get_boxes_for_runs", lambda: self.mongo_db[ws_name].find(filter_dict, fields_dict), return_records=True)
 
         console.diag("after get_boxes_for_runs()")        
         return run_records
@@ -327,8 +375,10 @@ class MongoDB():
         # cannot get "distinct" command to work ("command not supported")
         #cursor = db["__jobs__"].distinct("ws_name") 
 
-        cursor = self.mongo_with_retries("get_all_experiments_in_ws", lambda: self.mongo_db["__jobs__"].find({"ws_name": ws_name}, {"exper_name": 1}) )
-        exper_names = [rec["exper_name"] for rec in cursor if "exper_name" in rec]
+        records = self.mongo_with_retries("get_all_experiments_in_ws", lambda: self.mongo_db["__jobs__"].find({"ws_name": ws_name}, {"exper_name": 1}), 
+            return_records=True)
+
+        exper_names = [rec["exper_name"] for rec in records if "exper_name" in rec]
         exper_names = list(set(exper_names))   # remove dups
 
         console.diag("after get_all_experiments()")        
@@ -342,7 +392,7 @@ class MongoDB():
             - keep all runs for ws in cache and apply filter locally (TODO: this option)
         '''
         if include_log_records:
-            fields_dict = {"log_records": 1}
+            fields_dict = None   # all fields, including log_records
             fn_cache = self.run_cache_dir + "/" + constants.ALL_RUNS_CACHE_FN
         else:
             fields_dict = {"log_records": 0}
@@ -354,7 +404,7 @@ class MongoDB():
             last_count=last_count, sort_dict=sort_dict)
 
     def get_all_runs(self, aggregator_dest, ws_name, job_or_exper_name, filter_dict=None, fields_dict=None, use_cache=True, 
-        fn_cache=None, first_count=None, last_count=None, sort_dict=None):
+        fn_cache=None, first_count=None, last_count=None, sort_dict=None, batch_size=None):
         '''
         cache design: 
             - organized all cached run information by the way it was accessed: a folder for each workspace (created on demand), 
@@ -407,47 +457,32 @@ class MongoDB():
         #console.print("  mongo: filter: {}, fields: {}, sort: {}".format(filter_dict, fields_dict, sort_dict))
         console.diag("  mongo: filter: {}, fields: {}".format(filter_dict, fields_dict))
 
-        # limit query to avoid "message max exceeded" errors
-        max_query_records = 3000  
-        started = time.time()
 
         #records = self.mongo_db[ws_name].find(filter_dict, fields_dict)  
-        cmd_func = lambda: self.mongo_db[ws_name].find(filter_dict, fields_dict)
-        cursor = self.mongo_with_retries("get_all_runs", cmd_func)
+        def cmd_func():
+            started = time.time()
+    
+            cursor = self.mongo_db[ws_name].find(filter_dict, fields_dict)
 
-        # SORT TOTALLY BUSTED ON COSMOS: 
-        #   - sort of "-id" returns random order each time
-        #   - sort of "test-acc" returns 0 records (if ANY missing values, NO records returned)
-        #   - docs say pass a dict, but code wants list of 2-tuples (pymongo library)
+            #avail = cursor.count()
 
-        # if sort_dict:
-        #     items = list(sort_dict.items())
-        #     key, value = items[0]
-        #     import pymongo
-        #     cursor = cursor.sort("job", 1)  # key, value)
+            # break query into multiple calls to avoid "message max exceeded" errors
+            if batch_size:
+                cursor = cursor.batch_size(batch_size)
 
-        # adjust cursor per first_count, last_count
+            records = list(cursor)
 
-        # because SORT is busted, we can't use mongo for first/last either
-        # if last_count:
-        #     if last_count is True:
-        #         last_count = 25
-        #     avail = cursor.count()
-        #     skip_count = avail - last_count
-        #     if skip_count > 0:
-        #         cursor = cursor.skip(skip_count)
-        # elif first_count:
-        #     if first_count is True:
-        #         first_count = 25
-        #     cursor = cursor.limit(first_count)
+            if console.level in ["diagnostics", "detail"]:
+                elapsed = time.time() - started
+                total_count = self.mongo_db[ws_name].count()
+                console.diag("  mongo query returned {} records (of {}), took: {:2f} secs".format(len(return_count), total_count, elapsed))
 
-        records = list(cursor)
+            #explanation = cursor.explain()
 
+            return records
+
+        records = self.mongo_with_retries("get_all_runs", cmd_func)
         return_count = len(records)
-        total_count = self.mongo_db[ws_name].count()
-
-        elapsed = time.time() - started
-        console.diag("  mongo query returned {} records (of {}), took: {:2f} secs".format(return_count, total_count, elapsed))
 
         if cache:
             cache += records
@@ -463,23 +498,27 @@ class MongoDB():
         return records
 
     def update_run_info(self, ws_name, run_id, dd, clear=False, upsert=True):
-        if clear:
-            update_doc = { "$unset": dd}
-        else:
-            update_doc = { "$set": dd}
 
-        # update, create prop if needed
-        self.mongo_with_retries("update_run_info", lambda: self.mongo_db[ws_name].update_one( {"_id": run_id}, update_doc, upsert=upsert) )
+        if self.update_run_stats:
+            if clear:
+                update_doc = { "$unset": dd}
+            else:
+                update_doc = { "$set": dd}
+
+            # update, create prop if needed
+            self.mongo_with_retries("update_run_info", lambda: self.mongo_db[ws_name].update_one( {"_id": run_id}, update_doc, upsert=upsert) )
 
     def update_runs_from_filter(self, ws_name, filter, dd, clear=False, upsert=True):
-        if clear:
-            update_doc = { "$unset": dd}
-        else:
-            update_doc = { "$set": dd}
 
-        # update, create prop if needed
-        result = self.mongo_with_retries("update_runs_from_filter", lambda: self.mongo_db[ws_name].update_many( filter, update_doc, upsert=upsert) )
-        return result
+        if self.update_run_stats:
+            if clear:
+                update_doc = { "$unset": dd}
+            else:
+                update_doc = { "$set": dd}
+
+            # update, create prop if needed
+            result = self.mongo_with_retries("update_runs_from_filter", lambda: self.mongo_db[ws_name].update_many( filter, update_doc, upsert=upsert) )
+            return result
 
     #---- JOBS ----
 
@@ -487,37 +526,36 @@ class MongoDB():
         # does document for ws_name exist?
         job_id = "job1"
         cmd = lambda: self.mongo_db["__jobs__"].find({"_id": job_id}, {"_id": 1}).limit(1)
-        cursor = self.mongo_with_retries("does_jobs_exist", cmd)
-        found = bool(cursor and cursor.count)
+        records = self.mongo_with_retries("does_jobs_exist", cmd, return_records=True)
+        found = len(records) > 0
 
         return found
 
     def get_info_for_jobs(self, filter_dict, fields_dict=None):
 
-        cursor = self.mongo_with_retries("get_info_for_jobs", lambda: self.mongo_db["__jobs__"].find(filter_dict, fields_dict))
-        job_records = list(cursor) if cursor else []
+        job_records = self.mongo_with_retries("get_info_for_jobs", lambda: self.mongo_db["__jobs__"].find(filter_dict, fields_dict), return_records=True)
 
         console.diag("after get_info_for_jobs()")        
         return job_records
 
     def update_job_info(self, job_id, dd, clear=False, upsert=True):
-        if clear:
-            update_doc = { "$unset": dd}
-        else:
-            update_doc = { "$set": dd}
 
-        # update, create if needed
-        self.mongo_with_retries("update_job_info", lambda: self.mongo_db["__jobs__"].update_one( {"_id": job_id}, update_doc, upsert=upsert) )
+        if self.update_job_stats:
+            if clear:
+                update_doc = { "$unset": dd}
+            else:
+                update_doc = { "$set": dd}
+
+            # update, create if needed
+            self.mongo_with_retries("update_job_info", lambda: self.mongo_db["__jobs__"].update_one( {"_id": job_id}, update_doc, upsert=upsert) )
 
     def get_job_names(self, filter_dict=None):
         job_names = []
         fields_dict = {"_id": 1}
         
         cmd_func = lambda: self.mongo_db["__jobs__"].find(filter_dict, fields_dict)
-        cursor = self.mongo_with_retries("get_job_names", cmd_func)
-        if cursor:
-            jobs = list(cursor)
-
+        jobs = self.mongo_with_retries("get_job_names", cmd_func, return_records=True)
+        if jobs:
             # filter out just the names
             job_names = [ job["_id"] for job in jobs]
 
@@ -529,10 +567,10 @@ class MongoDB():
 
         # does document for ws_name exist?
         cmd = lambda: self.mongo_db["__jobs__"].find({"_id": job_id}, {"ws_name": 1}).limit(1)
-        cursor = self.mongo_with_retries("get_job_workspace", cmd)
+        records = self.mongo_with_retries("get_job_workspace", cmd, return_records=True)
 
-        if cursor and cursor.count():
-            result = list(cursor)[0]
+        if records:
+            result = records[0]
             if "ws_name" in result:
                 ws_name = result["ws_name"]
 
@@ -544,11 +582,12 @@ class MongoDB():
             - increment the job's "running_nodes" property
             - set the "job_status" property to "running"
         '''
-        cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {"running_nodes": 1} })
-        self.mongo_with_retries("job_node_start", cmd)
+        if self.update_job_stats:
+            cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {"running_nodes": 1} })
+            self.mongo_with_retries("job_node_start", cmd)
 
-        cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$set": {"job_status": "running"} })
-        self.mongo_with_retries("job_node_start", cmd)
+            cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$set": {"job_status": "running"} })
+            self.mongo_with_retries("job_node_start", cmd)
 
     def job_node_exit(self, job_id):
         '''
@@ -556,11 +595,12 @@ class MongoDB():
             - decrement the job's "running_nodes" property 
             - if running_nodes==0, set the "job_status" property to "completed"
         '''
-        cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {"running_nodes": -1} })
-        self.mongo_with_retries("job_node_exit", cmd)
+        if self.update_job_stats:
+            cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {"running_nodes": -1} })
+            self.mongo_with_retries("job_node_exit", cmd)
 
-        cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id, "running_nodes": 0}, update={"$set": {"job_status": "completed"} })
-        self.mongo_with_retries("job_node_exit", cmd)
+            cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id, "running_nodes": 0}, update={"$set": {"job_status": "completed"} })
+            self.mongo_with_retries("job_node_exit", cmd)
 
     def update_connect_info_by_node(self, job_id, node_id, connect_info):
         key = "connect_info_by_node." + node_id
@@ -572,8 +612,9 @@ class MongoDB():
         A job's run has started running.  We need to:
             - increment the job's "running_runs" property 
         '''
-        cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {"running_runs": 1} })
-        self.mongo_with_retries("job_run_start", cmd)
+        if self.update_job_stats:
+            cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {"running_runs": 1} })
+            self.mongo_with_retries("job_run_start", cmd)
 
     def job_run_exit(self, job_id, exit_code):
         '''
@@ -582,9 +623,10 @@ class MongoDB():
             - increment the job's "completed_runs" property
             - if exit_code != 0, increment the job's "error_runs" property
         '''
-        error_inc = 1 if exit_code else 0
-        cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {"running_runs": -1, "completed_runs": 1, "error_runs": error_inc} })
-        self.mongo_with_retries("job_run_exit", cmd)
+        if self.update_job_stats:
+            error_inc = 1 if exit_code else 0
+            cmd = lambda: self.mongo_db["__jobs__"].find_and_modify( {"_id": job_id}, update={"$inc": {"running_runs": -1, "completed_runs": 1, "error_runs": error_inc} })
+            self.mongo_with_retries("job_run_exit", cmd)
 
     def run_start(self, ws_name, run_name):
         '''
@@ -592,43 +634,45 @@ class MongoDB():
             - set the run "start_time" property to NOW
             - set the run "queue_duration" property to NOW - created_time
         '''
-        now = arrow.now()
-        now_str = str(now)
+        if self.update_run_stats:
+            now = arrow.now()
+            now_str = str(now)
 
-        # get create_time of run
-        cmd = lambda: self.mongo_db[ws_name].find({"_id": run_name}, {"create_time": 1})
-        cursor = self.mongo_with_retries("run_start", cmd)
+            # get create_time of run
+            cmd = lambda: self.mongo_db[ws_name].find({"_id": run_name}, {"create_time": 1})
+            records = self.mongo_with_retries("run_start", cmd, return_records=True)
 
-        doc = cursor.next()
-        if doc and "create_time" in doc:
-            create_time_str = doc["create_time"]
-            create_time = arrow.get(create_time_str)
+            doc = records[0] if records else None
+            if doc and "create_time" in doc:
+                create_time_str = doc["create_time"]
+                create_time = arrow.get(create_time_str)
 
-            # compute time in "queue" 
-            queue_duration = utils.time_diff(now, create_time)
+                # compute time in "queue" 
+                queue_duration = utils.time_diff(now, create_time)
 
-            cmd = lambda: self.mongo_db[ws_name].find_and_modify( {"_id": run_name}, update={"$set": {"start_time": now_str, "queue_duration": queue_duration} })
-            self.mongo_with_retries("run_start", cmd)
+                cmd = lambda: self.mongo_db[ws_name].find_and_modify( {"_id": run_name}, update={"$set": {"start_time": now_str, "queue_duration": queue_duration} })
+                self.mongo_with_retries("run_start", cmd)
 
     def run_exit(self, ws_name, run_name):
         '''
         A run has finished running.  We need to:
             - set the run "run_duration" property to NOW - start_time
         '''
-        now = arrow.now()
-        now_str = str(now)
+        if self.update_run_stats:
+            now = arrow.now()
+            now_str = str(now)
 
-        # get start_time of run
-        cmd = lambda: self.mongo_db[ws_name].find({"_id": run_name}, {"start_time": 1})
-        cursor = self.mongo_with_retries("run_exit", cmd)
+            # get start_time of run
+            cmd = lambda: self.mongo_db[ws_name].find({"_id": run_name}, {"start_time": 1})
+            records = self.mongo_with_retries("run_exit #1", cmd, return_records=True)
 
-        doc = cursor.next()
-        if doc and "start_time" in doc:
-            start_time_str = doc["start_time"]
-            start_time = arrow.get(start_time_str)
+            doc = records[0] if records else None
+            if doc and "start_time" in doc:
+                start_time_str = doc["start_time"]
+                start_time = arrow.get(start_time_str)
 
-            # compute run_duration 
-            run_duration = utils.time_diff(now, start_time)
+                # compute run_duration 
+                run_duration = utils.time_diff(now, start_time)
 
-            cmd = lambda: self.mongo_db[ws_name].find_and_modify( {"_id": run_name}, update={"$set": {"run_duration": run_duration} })
-            self.mongo_with_retries("run_exit", cmd)
+                cmd = lambda: self.mongo_db[ws_name].find_and_modify( {"_id": run_name}, update={"$set": {"run_duration": run_duration} })
+                self.mongo_with_retries("run_exit #2", cmd)
