@@ -107,9 +107,9 @@ class MongoRunIndex():
 
     def _get_job_property(self, prop_name):
         cmd = lambda: self.mongo.mongo_db["__jobs__"].find( {"_id": self.job_id}, {prop_name: 1})
-        cursor = self.mongo.mongo_with_retries("_get_job_property", cmd)
+        records = self.mongo.mongo_with_retries("_get_job_property", cmd, return_records=True)
 
-        value = utils.safe_cursor_value(cursor, prop_name)
+        value = utils.safe_cursor_value(records, prop_name)
         return value
 
     def _get_first_entry(self, filter, update):
