@@ -38,6 +38,13 @@ logger = logging.getLogger(__name__)
 # public function
 def store_from_context(context):
     store = Store(context.store_creds, provider_code_path=context.store_code_path, mongo_conn_str=context.mongo_conn_str)
+
+    # set switches for mongo stats, logging, and scheduling
+    mongo = store.mongo
+    mongo.update_job_stats = context.update_job_stats
+    mongo.update_run_stats = context.update_run_stats
+    mongo.add_log_records = context.add_log_records
+
     return store
 
 # main class that implements the XT STORE API
@@ -501,12 +508,16 @@ class Store():
         script = os.path.basename(path) if path else None
         create_time = utils.get_time()
 
+        build_parts = constants.BUILD.split(",")
+        xt_version = build_parts[0].split(":")[1].strip()
+        xt_build = build_parts[1].split(":")[1].strip()
+
         dd = {"ws": ws_name, "run_name": child_name, "run_num": run_num, "run_guid": run_guid, "description": description, "exper_name": exper_name, 
             "job_id": job_id, "node_index": node_index, "is_outer": False, "is_parent": False, "is_child": True,
             "from_ip": from_ip, "from_computer_name": from_host, "username": username, 
             "box_name": box_name, "app_name": app_name, "repeat": None, "path": path, "script": script, "create_time": create_time,
             "compute": compute, "service_type": service_type, "sku": sku, "search_style": search_style,
-            "search_type": search_type, "xt_version": constants.BUILD, "run_index": run_index}
+            "search_type": search_type,  "xt_build": xt_build, "xt_version": xt_version, "run_index": run_index}
 
         self.mongo.create_mongo_run(dd)
 
@@ -792,10 +803,10 @@ class Store():
         return self.mongo.get_ws_runs(ws_name, filter_dict, include_log_records, first_count, last_count, sort_dict)
 
     def get_all_runs(self, aggregator_dest, ws_name, job_or_exper_name, filter_dict=None, fields_dict=None, use_cache=False, 
-        fn_cache=None, first_count=None, last_count=None, sort_dict=None):
+        fn_cache=None, first_count=None, last_count=None, sort_dict=None, batch_size=None):
 
         return self.mongo.get_all_runs(aggregator_dest, ws_name, job_or_exper_name, filter_dict, fields_dict, use_cache,
-            fn_cache, first_count, last_count, sort_dict)
+            fn_cache, first_count, last_count, sort_dict, batch_size=batch_size)
 
     def wrapup_run(self, ws_name, run_name, aggregate_dest, dest_name, status, exit_code, primary_metric, maximize_metric, 
         report_rollup, rundir, after_files_list, log_events=True, capture_files=True, job_id=None, is_parent=False, 
