@@ -88,7 +88,7 @@ class StoreBlobObjs():
 
     def import_jobs_to_mongo_if_needed(self, mongo):
         console.diag("before mongo import check")
-        found = mongo.does_jobs_exist()
+        found = True   # mongo.does_jobs_exist()
         console.diag("after mongo import check")
 
         if not found:
@@ -960,6 +960,9 @@ class BlobsObject():
             exclude_dirs_and_files=exclude_dirs_and_files)
 
     def _process_parent_path(self, container, base_path, path):
+        if path.startswith("./"):
+            path = path[2:]
+
         while path.startswith("../"):
             if base_path:
                 base_path = os.path.dirname(base_path)
@@ -972,7 +975,7 @@ class BlobsObject():
 
     def _get_container_path_target(self, path):
         '''
-        This help function is the first step in converting all of our _xxx (internal)
+        This helper function is the first step in converting all of our _xxx (internal)
         methods to accept as parameters:
             - container  (optional - None or the Azure container name to use)
             - path       (optional - None, or the container-relative path to the target store object)
@@ -989,6 +992,23 @@ class BlobsObject():
 
         container = self.container
 
+        if path:
+            if path.startswith("$data"):
+
+                # expand $data
+                path = path.replace("$data", "/" + utils.DATA_STORE_ROOT)
+            elif path.startswith("$models"):
+
+                # expand $models
+                path = path.replace("$models", "/" + utils.MODELS_STORE_ROOT)
+            elif path.startswith("$runs"):
+
+                # expand $runs
+                self.base_path = "runs"
+                path = path.replace("$runs", "")
+                if path.startswith("/"):
+                    path = path[1:]
+
         if path:
             #console.print("path=", path)
             # path specified
