@@ -35,16 +35,28 @@ def get_fn_local_config(args):
     found_run = False
 
     for part in parts:
+        # don't change the order of these checks
         if found_run:
             if part.endswith(".yaml"):
                 fn = part
+            else:
+                # is there an XT confile file in the script dir?
+                fnx = os.path.join(os.path.dirname(part), constants.CONFIG_FN)
+                if os.path.exists(fnx):
+                    fn = os.path.relpath(fnx, ".")
+                    if os.path.dirname(fn):
+                        # only print if its not the current directory
+                        console.print("using script config file: {}".format(fn))
             break
+
         if not part.startswith("--"):
             if part == "run":
                 found_run = True
             else:
+                # some other command
                 break
 
+
     return fn
 
 def main(cmd=None, new_start_time=None, capture_output=False, mini=False, raise_syntax_exception=True):
@@ -116,24 +128,26 @@ def main(cmd=None, new_start_time=None, capture_output=False, mini=False, raise_
     if mini:
         # a dict of commands + arg/options to be surfaced (None means use all args/options)
         show_commands = {
-            "cancel_job": ["job"], 
+            "cancel_all": ["target"],
+            "cancel_job": ["job-id"], 
+            "cancel_run": ["run-names"],
             "clear_credentials": [],
             "config_cmd": ["default", "create"],
-            "run": ["script", "script-args", "experiment", "hp-config", "max-runs", "nodes", "runs", "search-type", "target"], 
-            "extract": ["run-name", "dest-dir", "browse"], 
-            "list_runs": ["run-list", "job", "experiment", "all", "first", "last", "filter", "sort", "reverse", "status", "available"],
-            "list_jobs": ["job-list", "experiment", "all", "first", "last", "filter", "sort", "reverse", "status", "available"],
-            #"list_experiments": ["name"],
-            "plot": ["runs", "col-list", "aggregate", "break-on", "cap-size", "error-bars", "group-by", "layout", 
-                "legend-titles", "marker-shape", "marker-size", "max-runs", "max-traces", "plot-titles", "save-to", "show-legend",
-                "smoothing-factor", "title", "x"],  
+            "create_demo": ["destination", "response"],
+            "create_services_template": [],
+            "download": ["store-path", "local-path", ],
+            "extract": ["runs", "dest-dir", "browse", "workspace"], 
             "help": ["command", "about", "browse", "version"],
             "help_topics": ["topic", "browse"],
-            "generate_help": ["dest-dir"],
-            "view_console": ["target", "workspace", "node-index"],
-            "create_demo": ["destination", "response"],
-            "create_services_template": []
-            #"root": ["help", "console", "stack-trace"]
+            "list_blobs": ["path"],
+            "list_jobs": ["job-list", "experiment", "all", "first", "last", "filter", "sort", "reverse", "status", "available"],
+            "list_runs": ["run-list", "job", "experiment", "all", "first", "last", "filter", "sort", "reverse", "status", "available"],
+            "monitor": ["name"],
+            "run": ["script", "script-args", "experiment", "hp-config", "max-runs", "nodes", "runs", "search-type", "target"],
+            "upload": ["local-path", "store-path"],
+            "view_console": ["name", "target", "workspace", "node-index"],
+            "view_metrics": ["runs", "metrics"],
+            "view_run": ["run-name"]
             }
 
         dispatcher.show_commands(show_commands)
@@ -167,3 +181,5 @@ def main(cmd=None, new_start_time=None, capture_output=False, mini=False, raise_
 
 if __name__ == "__main__":
     main()
+
+
