@@ -125,8 +125,8 @@ class XTVault():
             errors.syntax_error("unrecognized authentication type '{}'".format(authentication))
 
         new_creds = True
-        # outer_token = credential.get_token()
-        # token = outer_token.token
+        outer_token = credential.get_token()
+        token = outer_token.token
 
         # expires = outer_token[1]
         # elapsed = expires - time.time()
@@ -144,6 +144,8 @@ class XTVault():
         self.apply_creds(key_text)
         self.keys["xt_server_cert"] = xt_server_cert
 
+        self.keys["object_id"] = self.get_me_graph_property(token, "id")
+
         # return creds as json string
         return json.dumps(self.keys)
 
@@ -162,26 +164,26 @@ class XTVault():
         secret = secret_bundle.value
         return secret
 
+    def get_me_graph_property(self, token, property_name):
+        #console.print("get_user_principle_name: token=", token)
+
+        import requests
+        import json
+
+        endpoint =  "https://graph.microsoft.com/v1.0/me"
+        headers = {'Authorization': 'Bearer ' + token}
+
+        graph_data = requests.get(endpoint, headers=headers).json()
+        if "error" in graph_data:
+            error = graph_data["error"]
+            errors.config_error("{}: {}".format(error["code"], error["message"]))
+
+        #console.print("get_user_principle_name: graph_data=", graph_data)
+
+        upn = graph_data[property_name]
+        return upn
+
 # flat function 
-# def get_user_principle_name(token):
-#     #console.print("get_user_principle_name: token=", token)
-
-#     import requests
-#     import json
-
-#     endpoint =  "https://graph.microsoft.com/v1.0/me"
-#     headers = {'Authorization': 'Bearer ' + token}
-
-#     graph_data = requests.get(endpoint, headers=headers).json()
-#     if "error" in graph_data:
-#         error = graph_data["error"]
-#         errors.config_error("{}: {}".format(error["code"], error["message"]))
-
-#     #console.print("get_user_principle_name: graph_data=", graph_data)
-
-#     upn = graph_data["userPrincipalName"]
-#     return upn
-
 # def get_user_photo(token):
 #     import requests
 #     import json
